@@ -132,6 +132,7 @@
 #include "sock_ut.h"
 #include "async_task.h"
 #include "dset.h"
+#include "timer_proc.h"
 
 #ifdef DEBUG_DMALLOC
 #include <dmalloc.h>
@@ -200,7 +201,11 @@ Options:\n\
     -O nr        Script optimization level (debugging option)\n\
     -a mode      Auto aliases mode: enable with yes or on,\n\
                   disable with no or off\n\
-    -A define    Add config pre-processor define (e.g., -A WITH_AUTH)\n"
+    -A define    Add config pre-processor define (e.g., -A WITH_AUTH)\n\
+    -x name      Specify internal manager for shared memory (shm)\n\
+                  - can be: fm, qm or tlsf\n\
+    -X name      Specify internal manager for private memory (pkg)\n\
+                  - if omitted, the one for shm is used\n"
 #ifdef STATS
 "    -s file     File to which statistics is dumped (disabled otherwise)\n"
 #endif
@@ -497,6 +502,12 @@ char* pid_file = 0; /* filename as asked by use */
 char* pgid_file = 0;
 
 
+/* memory manager */
+#define SR_MEMMNG_DEFAULT	"fm"
+
+char *sr_memmng_pkg = NULL;
+char *sr_memmng_shm = NULL;
+
 /* call it before exiting; if show_status==1, mem status is displayed */
 void cleanup(show_status)
 {
@@ -504,7 +515,7 @@ void cleanup(show_status)
 	
 	/*clean-up*/
 #ifndef SHM_SAFE_MALLOC
-	if (mem_lock)
+	if (_shm_lock)
 		shm_unlock(); /* hack: force-unlock the shared memory lock in case
 					 some process crashed and let it locked; this will
 					 allow an almost gracious shutdown */
@@ -571,12 +582,12 @@ void cleanup(show_status)
 		}
 	}
 	/* zero all shmem alloc vars that we still use */
-	shm_mem_destroy();
+	shm_destroy_manager();
 #endif
 	destroy_lock_ops();
 	if (pid_file) unlink(pid_file);
 	if (pgid_file) unlink(pgid_file);
-	destroy_pkg_mallocs();
+	pkg_destroy_manager();
 }
 
 
@@ -742,10 +753,14 @@ void handle_sigs(void)
 			}
 			LM_INFO("terminating due to SIGCHLD\n");
 #endif
+			LM_DBG("terminating due to SIGCHLD\n");
 			/* exit */
 			shutdown_children(SIGTERM, 1);
-			LM_DBG("terminating due to SIGCHLD\n");
-			exit(0);
+			if (WIFSIGNALED(chld_status)) {
+				exit(1);
+			} else {
+				exit(0);
+			}
 			break;
 
 		case SIGHUP: /* ignoring it*/
@@ -1218,6 +1233,7 @@ int main_loop(void)
 	int r;
 #endif
 	int nrprocs;
+	int woneinit;
 
 	/* one "main" process and n children handling i/o */
 	if (dont_fork){
@@ -1287,6 +1303,7 @@ int main_loop(void)
 		cfg_register_child(
 				1   /* main = udp listener */
 				+ 1 /* timer */
+				+ 1 /* wtimer */
 #ifdef USE_SLOW_TIMER
 				+ 1 /* slow timer */
 #endif
@@ -1356,6 +1373,10 @@ int main_loop(void)
 				}else{
 				}
 
+		if(sr_wtimer_start()<0) {
+			LM_CRIT("Cannot start wtimer\n");
+			goto error;
+		}
 		/* main process, receive loop */
 		process_no=0; /*main process number*/
 		pt[process_no].pid=getpid();
@@ -1387,6 +1408,7 @@ int main_loop(void)
 		 * will be added later.) */
 		cfg_register_child(
 				1   /* timer */
+				+ 1   /* wtimer */
 #ifdef USE_SLOW_TIMER
 				+ 1 /* slow timer */
 #endif
@@ -1534,6 +1556,7 @@ int main_loop(void)
 		if (counters_prefork_init(get_max_procs()) == -1) goto error;
 
 
+		woneinit = 0;
 		/* udp processes */
 		for(si=udp_listen; si; si=si->next){
 			nrprocs = (si->workers>0)?si->workers:children_no;
@@ -1570,8 +1593,13 @@ int main_loop(void)
 #ifdef STATS
 					setstats( i+r*children_no );
 #endif
+					if(woneinit==0) {
+						if(run_child_one_init_route()<0)
+							goto error;
+					}
 					return udp_rcv_loop();
 				}
+				woneinit = 1;
 			}
 			/*parent*/
 			/*close(udp_sock)*/; /*if it's closed=>sendto invalid fd errors?*/
@@ -1643,6 +1671,10 @@ int main_loop(void)
 			if (arm_timer()<0) goto error;
 			timer_main();
 		}
+		if(sr_wtimer_start()<0) {
+			LM_CRIT("Cannot start wtimer\n");
+			goto error;
+		}
 
 	/* init childs with rank==MAIN before starting tcp main (in case they want
 	 * to fork  a tcp capable process, the corresponding tcp. comm. fds in
@@ -1671,7 +1703,7 @@ int main_loop(void)
 		}
 #endif
 		/* main */
-		strncpy(pt[0].desc, "attendant", MAX_PT_DESC );
+		strncpy(pt[0].desc, "main process - attendant", MAX_PT_DESC );
 #ifdef USE_TCP
 		close_extra_socks(PROC_ATTENDANT, get_proc_no());
 		if(!tcp_disable){
@@ -1767,6 +1799,7 @@ static int calc_proc_no(void)
 #ifdef USE_SLOW_TIMER
 		+ 1 /* slow timer process */
 #endif
+		+ 1 /* wtimer process */
 #ifdef USE_TCP
 		+((!tcp_disable)?( 1/* tcp main */ + tcp_listeners ):0)
 #endif
@@ -1793,6 +1826,7 @@ int main(int argc, char** argv)
 	int dont_fork_cnt;
 	struct name_lst* n_lst;
 	char *p;
+	struct stat st = {0};
 
 	/*init*/
 	time(&up_since);
@@ -1807,7 +1841,7 @@ int main(int argc, char** argv)
 	dprint_init_colors();
 
 	/* command line options */
-	options=  ":f:cm:M:dVIhEeb:l:L:n:vKrRDTN:W:w:t:u:g:P:G:SQ:O:a:A:"
+	options=  ":f:cm:M:dVIhEeb:l:L:n:vKrRDTN:W:w:t:u:g:P:G:SQ:O:a:A:x:X:"
 #ifdef STATS
 		"s:"
 #endif
@@ -1841,6 +1875,12 @@ int main(int argc, char** argv)
 						goto error;
 					};
 					break;
+			case 'x':
+					sr_memmng_shm = optarg;
+					break;
+			case 'X':
+					sr_memmng_pkg = optarg;
+					break;
 			default:
 					if (c == 'h' || (optarg && strcmp(optarg, "-h") == 0)) {
 						printf("version: %s\n", full_version);
@@ -1850,11 +1890,26 @@ int main(int argc, char** argv)
 					break;
 		}
 	}
-	
+
+	if(sr_memmng_pkg==NULL) {
+		if(sr_memmng_shm!=NULL) {
+			sr_memmng_pkg = sr_memmng_shm;
+		} else {
+			sr_memmng_pkg = SR_MEMMNG_DEFAULT;
+		}
+	}
+	if(sr_memmng_shm==NULL) {
+		sr_memmng_shm = SR_MEMMNG_DEFAULT;
+	}
+	shm_set_mname(sr_memmng_shm);
+	if (pkg_mem_size == 0) {
+		pkg_mem_size = PKG_MEM_POOL_SIZE;
+	}
+
 	/*init pkg mallocs (before parsing cfg or the rest of the cmd line !)*/
 	if (pkg_mem_size)
 		LM_INFO("private (per process) memory: %ld bytes\n", pkg_mem_size );
-	if (init_pkg_mallocs()==-1)
+	if (pkg_init_manager(sr_memmng_pkg)<0)
 		goto error;
 
 #ifdef DBG_MSG_QA
@@ -1868,11 +1923,17 @@ int main(int argc, char** argv)
 #ifdef USE_TCP
 	init_tcp_options(); /* set the defaults before the config */
 #endif
+
 	/* process command line (cfg. file path etc) */
 	optind = 1;  /* reset getopt */
 	/* switches required before script processing */
 	while((c=getopt(argc,argv,options))!=-1) {
 		switch(c) {
+			case 'M':
+			case 'x':
+			case 'X':
+					/* ignore, they were parsed immediately after startup */
+					break;
 			case 'f':
 					cfg_file=optarg;
 					break;
@@ -1893,10 +1954,6 @@ int main(int argc, char** argv)
 					};
 					LM_INFO("shared memory: %ld bytes\n", shm_mem_size );
 					break;
-			case 'M':
-					/* ignore it, it was parsed immediately after startup,
-					   the pkg mem. is already initialized at this point */
-					break;
 			case 'd':
 					/* ignore it, was parsed immediately after startup */
 					break;
@@ -1997,6 +2054,9 @@ int main(int argc, char** argv)
 					abort();
 		}
 	}
+	if (shm_mem_size == 0) {
+		shm_mem_size = SHM_MEM_POOL_SIZE;
+	}
 
 	if (endianness_sanity_check() != 0){
 		fprintf(stderr, "BUG: endianness sanity tests failed\n");
@@ -2312,6 +2372,21 @@ try_again:
 		}
 		sock_gid = gid;
 	}
+	/* create runtime dir if doesn't exist */
+	if (stat(runtime_dir, &st) == -1) {
+		if(mkdir(runtime_dir, 0700) == -1) {
+			LM_ERR("failed to create runtime dir %s\n", runtime_dir);
+			fprintf(stderr,  "failed to create runtime dir %s\n", runtime_dir);
+			goto error;
+		}
+		if(sock_uid!=-1 || sock_gid!=-1) {
+			if(chown(runtime_dir, sock_uid, sock_gid) == -1) {
+				LM_ERR("failed to change owner of runtime dir %s\n", runtime_dir);
+				fprintf(stderr,  "failed to change owner of runtime dir %s\n", runtime_dir);
+				goto error;
+			}
+		}
+	}
 	if (fix_all_socket_lists()!=0){
 		fprintf(stderr,  "failed to initialize list addresses\n");
 		goto error;
@@ -2357,6 +2432,8 @@ try_again:
 	if (!shm_initialized() && init_shm()<0)
 		goto error;
 #endif /* SHM_MEM */
+	pkg_print_manager();
+	shm_print_manager();
 	if (init_atomic_ops()==-1)
 		goto error;
 	if (init_basex() != 0){
@@ -2385,6 +2462,12 @@ try_again:
 		LM_CRIT("could not initialize timer, exiting...\n");
 		goto error;
 	}
+	/* init wtimer */
+	if(sr_wtimer_init()<0) {
+		LM_CRIT("could not initialize wtimer, exiting...\n");
+		goto error;
+	}
+
 #ifdef USE_DNS_CACHE
 	if (init_dns_cache()<0){
 		LM_CRIT("could not initialize the dns cache, exiting...\n");
