@@ -1,7 +1,7 @@
 /*
  * Copyright (C) 2001-2003 FhG Fokus
  *
- * This file is part of sip-router, a free SIP server.
+ * This file is part of Kamailio, a free SIP server.
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -16,30 +16,6 @@
  * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
-/*
- * History:
- * --------
- *              created by andrei
- *  2003-07-06  added fm_realloc (andrei)
- *  2004-07-19  fragments book keeping code and support for 64 bits
- *               memory blocks (64 bits machine & size >=2^32) 
- *              GET_HASH s/</<=/ (avoids waste of 1 hash cell)   (andrei)
- *  2004-11-10  support for > 4Gb mem., switched to long (andrei)
- *  2005-03-02  added fm_info() (andrei)
- *  2005-12-12  fixed realloc shrink real_used accounting (andrei)
- *              fixed initial size (andrei)
- *  2006-02-03  fixed realloc out of mem. free bug (andrei)
- *  2006-04-07  s/DBG/MDBG (andrei)
- *  2007-02-23  added fm_available() (andrei)
- *  2007-06-23  added hash bitmap (andrei)
- *  2009-09-28  added fm_sums() (patch from Dragos Vingarzan)
- *  2010-03-11  fix big fragments bug (smaller fragment was wrongly
- *               returned sometimes) (andrei)
- *  2010-03-12  fix real_used stats for realloc: a realloc that shrank an
- *               allocation accounted twice fro the frag. overhead (andrei)
- *  2010-09-30  fixed search for big fragments using the hash bitmap
- *               (only the first bucket was tried) (andrei)
- */
 
 /**
  * \file
@@ -48,7 +24,7 @@
  */
 
 
-#if !defined(q_malloc)  && (defined F_MALLOC)
+#if defined(F_MALLOC)
 
 #include <string.h>
 #include <stdlib.h>
@@ -72,7 +48,7 @@
 
 #define FRAG_OVERHEAD	(sizeof(struct fm_frag))
 #define INIT_OVERHEAD	\
-	(ROUNDUP(sizeof(struct fm_block))+sizeof(struct fm_frag))
+	(ROUNDUP(sizeof(struct fm_block))+2*sizeof(struct fm_frag))
 
 
 
@@ -115,6 +91,7 @@
 	((qm)->free_bitmap[(b)/FM_HASH_BMP_BITS] & (1UL<<((b)%FM_HASH_BMP_BITS)))
 
 
+#define fm_is_free(f) ((f)->is_free)
 
 /**
  * \brief Find the first free fragment in a memory block
@@ -182,14 +159,47 @@ inline static int fm_bmp_first_set(struct fm_block* qm, int start)
  * \name Memory manager boundary check pattern
  */
 /*@{ */
-#ifdef DBG_F_MALLOC
 #define ST_CHECK_PATTERN   0xf0f0f0f0 /** inserted at the beginning */
 #define END_CHECK_PATTERN1 0xc0c0c0c0 /** inserted at the end       */
 #define END_CHECK_PATTERN2 0xabcdefed /** inserted at the end       */
 /*@} */
-#endif
 
 
+/**
+ * \brief Extract memory fragment from free list
+ * \param qm memory block
+ * \param frag memory fragment
+ */
+static inline void fm_extract_free(struct fm_block* qm, struct fm_frag* frag)
+{
+	int hash;
+
+	hash = GET_HASH(frag->size);
+
+	if(frag->prev_free) {
+		frag->prev_free->next_free = frag->next_free;
+	} else {
+		qm->free_hash[hash].first = frag->next_free;
+	}
+	if(frag->next_free) {
+		frag->next_free->prev_free = frag->prev_free;
+	}
+
+	frag->prev_free = NULL;
+	frag->next_free = NULL;
+	frag->is_free = 0;
+
+	qm->ffrags--;
+	qm->free_hash[hash].no--;
+#ifdef F_MALLOC_HASH_BITMAP
+	if (qm->free_hash[hash].no==0)
+		fm_bmp_reset(qm, hash);
+#endif /* F_MALLOC_HASH_BITMAP */
+
+	qm->real_used+=frag->size;
+	qm->used+=frag->size;
+}
+
 /**
  * \brief Insert a memory fragment in a memory block
  * \param qm memory block
@@ -197,26 +207,49 @@ inline static int fm_bmp_first_set(struct fm_block* qm, int start)
  */
 static inline void fm_insert_free(struct fm_block* qm, struct fm_frag* frag)
 {
-	struct fm_frag** f;
+	struct fm_frag* f;
+	struct fm_frag* p;
 	int hash;
 	
 	hash=GET_HASH(frag->size);
-	f=&(qm->free_hash[hash].first);
+	f=qm->free_hash[hash].first;
+	p=NULL;
 	if (frag->size > F_MALLOC_OPTIMIZE){ /* because of '<=' in GET_HASH,
 											(different from 0.8.1[24] on
 											 purpose --andrei ) */
-		for(; *f; f=&((*f)->u.nxt_free)){
-			if (frag->size <= (*f)->size) break;
+		/* large fragments list -- add at a position ordered by size */
+		for(; f; f=f->next_free){
+			if (frag->size <= f->size) break;
+			p = f;
 		}
-	}
 	
-	/*insert it here*/
-	frag->u.nxt_free=*f;
-	*f=frag;
+		frag->next_free = f;
+		frag->prev_free = p;
+		if(f) {
+			f->prev_free = frag;
+		}
+		if(p) {
+			p->next_free = frag;
+		} else {
+			qm->free_hash[hash].first = frag;
+		}
+	} else {
+		/* fixed fragment size list -- add first */
+		frag->prev_free = 0;
+		frag->next_free = f;
+		if(f) {
+			f->prev_free = frag;
+		}
+		qm->free_hash[hash].first = frag;
+	}
+	frag->is_free = 1;
+	qm->ffrags++;
 	qm->free_hash[hash].no++;
 #ifdef F_MALLOC_HASH_BITMAP
 	fm_bmp_set(qm, hash);
 #endif /* F_MALLOC_HASH_BITMAP */
+	qm->used-=frag->size;
+	qm->real_used-=frag->size;
 }
 
 
@@ -232,7 +265,8 @@ static inline
 #ifdef DBG_F_MALLOC 
 void fm_split_frag(struct fm_block* qm, struct fm_frag* frag,
 					unsigned long size,
-					const char* file, const char* func, unsigned int line)
+					const char* file, const char* func, unsigned int line,
+					const char* mname)
 #else
 void fm_split_frag(struct fm_block* qm, struct fm_frag* frag,
 					unsigned long size)
@@ -253,20 +287,16 @@ void fm_split_frag(struct fm_block* qm, struct fm_frag* frag,
 		n=FRAG_NEXT(frag);
 		n->size=rest-FRAG_OVERHEAD;
 		FRAG_CLEAR_USED(n); /* never used */
-#if defined(DBG_F_MALLOC) || defined(MALLOC_STATS)
-		qm->real_used+=FRAG_OVERHEAD;
-#ifdef MALLOC_STATS
-		sr_event_exec(SREV_PKG_SET_REAL_USED, (void*)qm->real_used);
-#endif
-#endif
 #ifdef DBG_F_MALLOC
 		/* frag created by malloc, mark it*/
 		n->file=file;
-		n->func="frag. from fm_malloc";
+		n->func="frag. from fm_split_frag";
 		n->line=line;
-		n->check=ST_CHECK_PATTERN;
+		n->mname=mname;
 #endif
+		n->check=ST_CHECK_PATTERN;
 		/* reinsert n in free list*/
+		qm->used-=FRAG_OVERHEAD;
 		fm_insert_free(qm, n);
 	}else{
 		/* we cannot split this fragment any more => alloc all of it*/
@@ -280,7 +310,7 @@ void fm_split_frag(struct fm_block* qm, struct fm_frag* frag,
  * \param size Size of allocation
  * \return return the fm_block
  */
-struct fm_block* fm_malloc_init(char* address, unsigned long size)
+struct fm_block* fm_malloc_init(char* address, unsigned long size, int type)
 {
 	char* start;
 	char* end;
@@ -312,31 +342,75 @@ struct fm_block* fm_malloc_init(char* address, unsigned long size)
 	qm=(struct fm_block*)start;
 	memset(qm, 0, sizeof(struct fm_block));
 	qm->size=size;
-#if defined(DBG_F_MALLOC) || defined(MALLOC_STATS)
-	qm->real_used=init_overhead;
-	qm->max_real_used=qm->real_used;
-#endif
+	qm->used = size - init_overhead;
+	qm->real_used=size;
+	qm->max_real_used=init_overhead;
+	qm->type = type;
 	size-=init_overhead;
 	
 	qm->first_frag=(struct fm_frag*)(start+ROUNDUP(sizeof(struct fm_block)));
 	qm->last_frag=(struct fm_frag*)(end-sizeof(struct fm_frag));
-	/* init initial fragment*/
+	/* init first fragment*/
 	qm->first_frag->size=size;
+	qm->first_frag->prev_free=0;
+	qm->first_frag->next_free=0;
+	qm->first_frag->is_free=0;
+	/* init last fragment*/
 	qm->last_frag->size=0;
+	qm->last_frag->prev_free=0;
+	qm->last_frag->next_free=0;
+	qm->last_frag->is_free=0;
 	
-#ifdef DBG_F_MALLOC
 	qm->first_frag->check=ST_CHECK_PATTERN;
 	qm->last_frag->check=END_CHECK_PATTERN1;
-#endif
 	
 	/* link initial fragment into the free list*/
 	
 	fm_insert_free(qm, qm->first_frag);
-	
-	
+
 	return qm;
 }
 
+/**
+ * \brief Try merging free fragments to fit requested size
+ * \param qm memory block
+ * \param size memory allocation size
+ * \return address of allocated memory
+ */
+struct fm_frag* fm_search_defrag(struct fm_block* qm, unsigned long size)
+{
+	struct fm_frag* frag;
+	struct fm_frag* nxt;
+
+	frag = qm->first_frag;
+	while((char*)frag < (char*)qm->last_frag) {
+		nxt = FRAG_NEXT(frag);
+
+		if ( ((char*)nxt < (char*)qm->last_frag) && fm_is_free(frag)
+				&& fm_is_free(nxt)) {
+			/* join frag with all next consecutive free frags */
+			fm_extract_free(qm, frag);
+			do {
+				fm_extract_free(qm, nxt);
+				frag->size += nxt->size + FRAG_OVERHEAD;
+
+				/* after join - one frag less, add its overhead to used
+				 * (real_used already has it - f and n were extracted */
+				qm->used += FRAG_OVERHEAD;
+
+				if( frag->size >size )
+					return frag;
+
+				nxt = FRAG_NEXT(frag);
+			} while (((char*)nxt < (char*)qm->last_frag) && fm_is_free(nxt));
+
+			fm_insert_free(qm, frag);
+		}
+		frag = nxt;
+	}
+
+	return 0;
+}
 
 /**
  * \brief Main memory manager allocation function
@@ -347,15 +421,18 @@ struct fm_block* fm_malloc_init(char* address, unsigned long size)
  * \return address of allocated memory
  */
 #ifdef DBG_F_MALLOC
-void* fm_malloc(struct fm_block* qm, unsigned long size,
-					const char* file, const char* func, unsigned int line)
+void* fm_malloc(void* qmp, unsigned long size, const char* file,
+                    const char* func, unsigned int line, const char* mname)
 #else
-void* fm_malloc(struct fm_block* qm, unsigned long size)
+void* fm_malloc(void* qmp, unsigned long size)
 #endif
 {
-	struct fm_frag** f;
+	struct fm_block* qm;
+	struct fm_frag* f;
 	struct fm_frag* frag;
 	int hash;
+
+	qm = (struct fm_block*)qmp;
 	
 #ifdef DBG_F_MALLOC
 	MDBG("fm_malloc(%p, %lu) called from %s: %s(%d)\n", qm, size, file, func,
@@ -365,8 +442,6 @@ void* fm_malloc(struct fm_block* qm, unsigned long size)
 	if(unlikely(size==0)) size=4;
 	/*size must be a multiple of 8*/
 	size=ROUNDUP(size);
-/*	if (size>(qm->size-qm->real_used)) return 0; */
-
 	
 	/*search for a suitable free frag*/
 
@@ -374,10 +449,18 @@ void* fm_malloc(struct fm_block* qm, unsigned long size)
 	hash=fm_bmp_first_set(qm, GET_HASH(size));
 	if (likely(hash>=0)){
 		if (likely(hash<=F_MALLOC_OPTIMIZE/ROUNDTO)) { /* return first match */
-			f=&(qm->free_hash[hash].first);
-			goto found;
+			f=qm->free_hash[hash].first;
+			if(likely(f)) goto found;
+#ifdef DBG_F_MALLOC
+			MDBG(" block %p hash %d empty but no. is %lu\n", qm,
+					hash, qm->free_hash[hash].no);
+#endif
+			/* reset slot and try next hash */
+			qm->free_hash[hash].no=0;
+			fm_bmp_reset(qm, hash);
+			hash++;
 		}
-		/* if we are here we are searching for a "big" fragment
+		/* if we are here we are searching next hash slot or a "big" fragment
 		   between F_MALLOC_OPTIMIZE/ROUNDTO+1
 		   and F_MALLOC_OPTIMIZE/ROUNDTO + (32|64) - F_MALLOC_OPTIMIZE_FACTOR
 		   => 18 hash buckets on 32 bits and 50 buckets on 64 bits
@@ -385,108 +468,87 @@ void* fm_malloc(struct fm_block* qm, unsigned long size)
 		   hash buckets.
 		*/
 		do {
-			for(f=&(qm->free_hash[hash].first);(*f); f=&((*f)->u.nxt_free))
-				if ((*f)->size>=size) goto found;
+			for(f=qm->free_hash[hash].first; f; f=f->next_free)
+				if (f->size>=size) goto found;
 			hash++; /* try in next hash cell */
 		}while((hash < F_HASH_SIZE) &&
 				((hash=fm_bmp_first_set(qm, hash)) >= 0));
 	}
 #else /* F_MALLOC_HASH_BITMAP */
 	for(hash=GET_HASH(size);hash<F_HASH_SIZE;hash++){
-		f=&(qm->free_hash[hash].first);
-#if 0
-		if (likely(hash<=F_MALLOC_OPTIMIZE/ROUNDTO)) /* return first match */
-				goto found; 
-#endif
-		for(;(*f); f=&((*f)->u.nxt_free))
-			if ((*f)->size>=size) goto found;
+		f=qm->free_hash[hash].first;
+		for(; f; f=f->u.nxt_free)
+			if (f->size>=size) goto found;
 		/* try in a bigger bucket */
 	}
 #endif /* F_MALLOC_HASH_BITMAP */
-	/* not found, bad! */
+	/* not found, search by defrag */
+
+	frag = fm_search_defrag(qm, size);
+
+	if(frag) goto finish;
+
 	return 0;
 
 found:
 	/* we found it!*/
 	/* detach it from the free list*/
-	frag=*f;
-	*f=frag->u.nxt_free;
-	frag->u.nxt_free=0; /* mark it as 'taken' */
-	qm->free_hash[hash].no--;
-#ifdef F_MALLOC_HASH_BITMAP
-	if (qm->free_hash[hash].no==0)
-		fm_bmp_reset(qm, hash);
-#endif /* F_MALLOC_HASH_BITMAP */
-	
-	/*see if we'll use full frag, or we'll split it in 2*/
-	
-#ifdef DBG_F_MALLOC
-	fm_split_frag(qm, frag, size, file, func, line);
+	frag=f;
+	fm_extract_free(qm, frag);
 
-	frag->file=file;
-	frag->func=func;
-	frag->line=line;
-	frag->check=ST_CHECK_PATTERN;
-	MDBG("fm_malloc(%p, %lu) returns address %p \n", qm, size,
-		(char*)frag+sizeof(struct fm_frag));
+	/*see if use full frag or split it in two*/
+#ifdef DBG_F_MALLOC
+	fm_split_frag(qm, frag, size, file, func, line, mname);
 #else
 	fm_split_frag(qm, frag, size);
 #endif
-#if defined(DBG_F_MALLOC) || defined(MALLOC_STATS)
-	qm->real_used+=frag->size;
-	qm->used+=frag->size;
+
+finish:
+
+#ifdef DBG_F_MALLOC
+	frag->file=file;
+	frag->func=func;
+	frag->mname=mname;
+	frag->line=line;
+	MDBG("fm_malloc(%p, %lu) returns address %p \n", qm, size,
+		(char*)frag+sizeof(struct fm_frag));
+#endif
+	frag->check=ST_CHECK_PATTERN;
+
 	if (qm->max_real_used<qm->real_used)
 		qm->max_real_used=qm->real_used;
-#ifdef MALLOC_STATS
-	sr_event_exec(SREV_PKG_SET_USED, (void*)qm->used);
-	sr_event_exec(SREV_PKG_SET_REAL_USED, (void*)qm->real_used);
-#endif
-#endif
 	FRAG_MARK_USED(frag); /* mark it as used */
+	if(qm->type==MEM_TYPE_PKG) {
+		sr_event_exec(SREV_PKG_UPDATE_STATS, 0);
+	}
 	return (char*)frag+sizeof(struct fm_frag);
 }
 
 
 #ifdef MEM_JOIN_FREE
 /**
- * join fragment f with next one (if it is free)
+ * join fragment free frag f with next one (if it is free)
  */
 static void fm_join_frag(struct fm_block* qm, struct fm_frag* f)
 {
-	int hash;
-	struct fm_frag **pf;
-	struct fm_frag* n;
+	struct fm_frag *n;
 
 	n=FRAG_NEXT(f);
-	/* check if valid and if in free list */
-	if (((char*)n >= (char*)qm->last_frag) || (n->u.nxt_free==NULL))
+
+	/* check if n is valid and if in free list */
+	if (((char*)n >= (char*)qm->last_frag) || !fm_is_free(n))
 		return;
 
 	/* detach n from the free list */
-	hash=GET_HASH(n->size);
-	pf=&(qm->free_hash[hash].first);
-	/* find it */
-	for(;(*pf)&&(*pf!=n); pf=&((*pf)->u.nxt_free)); /*FIXME slow */
-	if (*pf==0){
-		/* not found, bad! */
-		LM_WARN("could not find %p in free list (hash=%ld)\n", n, GET_HASH(n->size));
-		return;
-	}
-	/* detach */
-	*pf=n->u.nxt_free;
-	qm->free_hash[hash].no--;
-#ifdef F_MALLOC_HASH_BITMAP
-	if (qm->free_hash[hash].no==0)
-		fm_bmp_reset(qm, hash);
-#endif /* F_MALLOC_HASH_BITMAP */
-	/* join */
+	fm_extract_free(qm, n);
+
+	/* join - f extended with size of n plus its overhead */
 	f->size+=n->size+FRAG_OVERHEAD;
-#if defined(DBG_F_MALLOC) || defined(MALLOC_STATS)
-	qm->real_used-=FRAG_OVERHEAD;
-#ifdef MALLOC_STATS
-	sr_event_exec(SREV_PKG_SET_REAL_USED, (void*)qm->real_used);
-#endif /* MALLOC_STATS */
-#endif /* DBG_F_MALLOC || MALLOC_STATS*/
+
+	/* after join - one frag less, add its overhead to used
+	 * (real_used already has it - f and n were extracted */
+	qm->used += FRAG_OVERHEAD;
+
 }
 #endif /*MEM_JOIN_FREE*/
 
@@ -498,14 +560,16 @@ static void fm_join_frag(struct fm_block* qm, struct fm_frag* f)
  * \param p freed memory
  */
 #ifdef DBG_F_MALLOC
-void fm_free(struct fm_block* qm, void* p, const char* file, const char* func, 
-				unsigned int line)
+void fm_free(void* qmp, void* p, const char* file, const char* func,
+				unsigned int line, const char* mname)
 #else
-void fm_free(struct fm_block* qm, void* p)
+void fm_free(void* qmp, void* p)
 #endif
 {
+	struct fm_block* qm;
 	struct fm_frag* f;
-	unsigned long size;
+
+	qm = (struct fm_block*)qmp;
 
 #ifdef DBG_F_MALLOC
 	MDBG("fm_free(%p, %p), called from %s: %s(%d)\n", qm, p, file, func, line);
@@ -529,24 +593,19 @@ void fm_free(struct fm_block* qm, void* p)
 	MDBG("fm_free: freeing block alloc'ed from %s: %s(%ld)\n",
 			f->file, f->func, f->line);
 #endif
-	if(unlikely(f->u.nxt_free!=NULL)) {
+	if(unlikely(fm_is_free(f))) {
 		LM_INFO("freeing a free fragment (%p/%p) - ignore\n",
 				f, p);
 		return;
 	}
-	size=f->size;
-#if defined(DBG_F_MALLOC) || defined(MALLOC_STATS)
-	qm->used-=size;
-	qm->real_used-=size;
-#ifdef MALLOC_STATS
-	sr_event_exec(SREV_PKG_SET_USED, (void*)qm->used);
-	sr_event_exec(SREV_PKG_SET_REAL_USED, (void*)qm->real_used);
-#endif
-#endif
+	if(qm->type==MEM_TYPE_PKG) {
+		sr_event_exec(SREV_PKG_UPDATE_STATS, 0);
+	}
 #ifdef DBG_F_MALLOC
 	f->file=file;
 	f->func=func;
 	f->line=line;
+	f->mname=mname;
 #endif
 #ifdef MEM_JOIN_FREE
 	if(unlikely(cfg_get(core, core_cfg, mem_join)!=0))
@@ -566,20 +625,22 @@ void fm_free(struct fm_block* qm, void* p)
  * \return reallocated memory block
  */
 #ifdef DBG_F_MALLOC
-void* fm_realloc(struct fm_block* qm, void* p, unsigned long size,
-					const char* file, const char* func, unsigned int line)
+void* fm_realloc(void* qmp, void* p, unsigned long size,
+					const char* file, const char* func, unsigned int line,
+					const char *mname)
 #else
-void* fm_realloc(struct fm_block* qm, void* p, unsigned long size)
+void* fm_realloc(void* qmp, void* p, unsigned long size)
 #endif
 {
+	struct fm_block* qm;
 	struct fm_frag *f;
-	struct fm_frag **pf;
 	unsigned long diff;
 	unsigned long orig_size;
 	struct fm_frag *n;
 	void *ptr;
-	int hash;
-	
+
+	qm = (struct fm_block*)qmp;
+
 #ifdef DBG_F_MALLOC
 	MDBG("fm_realloc(%p, %p, %lu) called from %s: %s(%d)\n", qm, p, size,
 			file, func, line);
@@ -592,7 +653,7 @@ void* fm_realloc(struct fm_block* qm, void* p, unsigned long size)
 	if (size==0) {
 		if (p)
 #ifdef DBG_F_MALLOC
-			fm_free(qm, p, file, func, line);
+			fm_free(qm, p, file, func, line, mname);
 #else
 			fm_free(qm, p);
 #endif
@@ -600,7 +661,7 @@ void* fm_realloc(struct fm_block* qm, void* p, unsigned long size)
 	}
 	if (p==0)
 #ifdef DBG_F_MALLOC
-		return fm_malloc(qm, size, file, func, line);
+		return fm_malloc(qm, size, file, func, line, mname);
 #else
 		return fm_malloc(qm, size);
 #endif
@@ -615,19 +676,9 @@ void* fm_realloc(struct fm_block* qm, void* p, unsigned long size)
 		/* shrink */
 #ifdef DBG_F_MALLOC
 		MDBG("fm_realloc: shrinking from %lu to %lu\n", f->size, size);
-		fm_split_frag(qm, f, size, file, "frag. from fm_realloc", line);
+		fm_split_frag(qm, f, size, file, "frag. from fm_realloc", line, mname);
 #else
 		fm_split_frag(qm, f, size);
-#endif
-#if defined(DBG_F_MALLOC) || defined(MALLOC_STATS)
-		/* fm_split frag already adds FRAG_OVERHEAD for the newly created
-		   free frag, so here we only need orig_size-f->size for real used */
-		qm->real_used-=(orig_size-f->size);
-		qm->used-=(orig_size-f->size);
-#ifdef MALLOC_STATS
-		sr_event_exec(SREV_PKG_SET_USED, (void*)qm->used);
-		sr_event_exec(SREV_PKG_SET_REAL_USED, (void*)qm->real_used);
-#endif
 #endif
 	}else if (f->size<size){
 		/* grow */
@@ -636,68 +687,40 @@ void* fm_realloc(struct fm_block* qm, void* p, unsigned long size)
 #endif
 		diff=size-f->size;
 		n=FRAG_NEXT(f);
+		/*if next frag is free, check if a join has enough size*/
 		if (((char*)n < (char*)qm->last_frag) && 
-				(n->u.nxt_free)&&((n->size+FRAG_OVERHEAD)>=diff)){
-			/* join  */
+				fm_is_free(n) && ((n->size+FRAG_OVERHEAD)>=diff)){
 			/* detach n from the free list */
-			hash=GET_HASH(n->size);
-			pf=&(qm->free_hash[hash].first);
-			/* find it */
-			for(;(*pf)&&(*pf!=n); pf=&((*pf)->u.nxt_free)); /*FIXME slow */
-			if (*pf==0){
-				/* not found, bad! */
-				LOG(L_CRIT, "BUG: fm_realloc: could not find %p in free "
-						"list (hash=%ld)\n", n, GET_HASH(n->size));
-				abort();
-			}
-			/* detach */
-			*pf=n->u.nxt_free;
-			qm->free_hash[hash].no--;
-#ifdef F_MALLOC_HASH_BITMAP
-			if (qm->free_hash[hash].no==0)
-				fm_bmp_reset(qm, hash);
-#endif /* F_MALLOC_HASH_BITMAP */
+			fm_extract_free(qm, n);
 			/* join */
 			f->size+=n->size+FRAG_OVERHEAD;
-		#if defined(DBG_F_MALLOC) || defined(MALLOC_STATS)
-			qm->real_used-=FRAG_OVERHEAD;
-#ifdef MALLOC_STATS
-			sr_event_exec(SREV_PKG_SET_REAL_USED, (void*)qm->real_used);
-#endif
-		#endif
+			qm->used+=FRAG_OVERHEAD;
+
 			/* split it if necessary */
 			if (f->size > size){
 		#ifdef DBG_F_MALLOC
 				fm_split_frag(qm, f, size, file, "fragm. from fm_realloc",
-						line);
+						line, mname);
 		#else
 				fm_split_frag(qm, f, size);
 		#endif
 			}
-		#if defined(DBG_F_MALLOC) || defined(MALLOC_STATS)
-			qm->real_used+=(f->size-orig_size);
-			qm->used+=(f->size-orig_size);
-#ifdef MALLOC_STATS
-			sr_event_exec(SREV_PKG_SET_USED, (void*)qm->used);
-			sr_event_exec(SREV_PKG_SET_REAL_USED, (void*)qm->real_used);
-#endif
-		#endif
 		}else{
 			/* could not join => realloc */
 	#ifdef DBG_F_MALLOC
-			ptr=fm_malloc(qm, size, file, func, line);
+			ptr=fm_malloc(qm, size, file, func, line, mname);
 	#else
 			ptr=fm_malloc(qm, size);
 	#endif
 			if (ptr){
 				/* copy, need by libssl */
 				memcpy(ptr, p, orig_size);
-	#ifdef DBG_F_MALLOC
-				fm_free(qm, p, file, func, line);
-	#else
-				fm_free(qm, p);
-	#endif
 			}
+	#ifdef DBG_F_MALLOC
+			fm_free(qm, p, file, func, line, mname);
+	#else
+			fm_free(qm, p);
+	#endif
 			p=ptr;
 		}
 	}else{
@@ -710,6 +733,9 @@ void* fm_realloc(struct fm_block* qm, void* p, unsigned long size)
 #ifdef DBG_F_MALLOC
 	MDBG("fm_realloc: returning %p\n", p);
 #endif
+	if(qm->type==MEM_TYPE_PKG) {
+		sr_event_exec(SREV_PKG_UPDATE_STATS, 0);
+	}
 	return p;
 }
 
@@ -718,8 +744,9 @@ void* fm_realloc(struct fm_block* qm, void* p, unsigned long size)
  * \brief Report internal memory manager status
  * \param qm memory block
  */
-void fm_status(struct fm_block* qm)
+void fm_status(void* qmp)
 {
+	struct fm_block* qm;
 	struct fm_frag* f;
 	int i,j;
 	int h;
@@ -728,6 +755,8 @@ void fm_status(struct fm_block* qm)
 	int memlog;
 	int mem_summary;
 
+	qm = (struct fm_block*)qmp;
+
 	memlog=cfg_get(core, core_cfg, memlog);
 	mem_summary=cfg_get(core, core_cfg, mem_summary);
 	LOG_(DEFAULT_FACILITY, memlog, "fm_status: ", "fm_status (%p):\n", qm);
@@ -764,7 +793,7 @@ void fm_status(struct fm_block* qm)
 	for(h=0,i=0,size=0;h<F_HASH_SIZE;h++){
 		unused=0;
 		for (f=qm->free_hash[h].first,j=0; f;
-				size+=f->size,f=f->u.nxt_free,i++,j++){
+				size+=f->size,f=f->next_free,i++,j++){
 			if (!FRAG_WAS_USED(f)){
 				unused++;
 #ifdef DBG_F_MALLOC
@@ -816,45 +845,19 @@ void fm_status(struct fm_block* qm)
  * \param qm memory block
  * \param info memory information
  */
-void fm_info(struct fm_block* qm, struct mem_info* info)
+void fm_info(void* qmp, struct mem_info* info)
 {
-	int r;
-	long total_frags;
-#if !defined(DBG_F_MALLOC) && !defined(MALLOC_STATS)
-	struct fm_frag* f;
-#endif
-	
+	struct fm_block* qm;
+
+	qm = (struct fm_block*)qmp;
 	memset(info,0, sizeof(*info));
-	total_frags=0;
 	info->total_size=qm->size;
 	info->min_frag=MIN_FRAG_SIZE;
-#if defined(DBG_F_MALLOC) || defined(MALLOC_STATS)
 	info->free=qm->size-qm->real_used;
 	info->used=qm->used;
 	info->real_used=qm->real_used;
 	info->max_used=qm->max_real_used;
-	for(r=0;r<F_HASH_SIZE; r++){
-		total_frags+=qm->free_hash[r].no;
-	}
-#else
-	/* we'll have to compute it all */
-	for (r=0; r<=F_MALLOC_OPTIMIZE/ROUNDTO; r++){
-		info->free+=qm->free_hash[r].no*UN_HASH(r);
-		total_frags+=qm->free_hash[r].no;
-	}
-	for(;r<F_HASH_SIZE; r++){
-		total_frags+=qm->free_hash[r].no;
-		for(f=qm->free_hash[r].first;f;f=f->u.nxt_free){
-			info->free+=f->size;
-		}
-	}
-	info->real_used=info->total_size-info->free;
-	info->used=0; /* we don't really now */
-	info->used=info->real_used-total_frags*FRAG_OVERHEAD-INIT_OVERHEAD-
-					FRAG_OVERHEAD;
-	info->max_used=0; /* we don't really now */
-#endif
-	info->total_frags=total_frags;
+	info->total_frags=qm->ffrags;
 }
 
 
@@ -864,32 +867,17 @@ void fm_info(struct fm_block* qm, struct mem_info* info)
  * \return Returns how much free memory is available, on error (not compiled
  * with bookkeeping code) returns (unsigned long)(-1)
  */
-unsigned long fm_available(struct fm_block* qm)
+unsigned long fm_available(void* qmp)
 {
+	struct fm_block* qm;
 
-#if defined(DBG_F_MALLOC) || defined(MALLOC_STATS)
+	qm = (struct fm_block*)qmp;
 	return qm->size-qm->real_used;
-#else
-	/* we don't know how much free memory we have and it's to expensive
-	 * to compute it */
-	return ((unsigned long)-1);
-#endif
 }
 
 
 #ifdef DBG_F_MALLOC
 
-typedef struct _mem_counter{
-	const char *file;
-	const char *func;
-	unsigned long line;
-	
-	unsigned long size;
-	int count;
-	
-	struct _mem_counter *next;
-} mem_counter;
-
 static mem_counter* get_mem_counter(mem_counter **root,struct fm_frag* f)
 {
 	mem_counter *x;
@@ -903,6 +891,7 @@ make_new:
 	x->file = f->file;
 	x->func = f->func;
 	x->line = f->line;
+	x->mname = f->mname;
 	x->count = 0;
 	x->size = 0;
 	x->next = *root;
@@ -915,11 +904,13 @@ make_new:
  * \brief Debugging helper, summary and logs all allocated memory blocks
  * \param qm memory block
  */
-void fm_sums(struct fm_block* qm)
+void fm_sums(void* qmp)
 {
+	struct fm_block* qm;
+
+	qm = (struct fm_block*)qmp;
 	struct fm_frag* f;
-	struct fm_frag* free_frag;
-	int i, hash;
+	int i;
 	int memlog;
 	mem_counter *root,*x;
 	
@@ -932,19 +923,10 @@ void fm_sums(struct fm_block* qm)
 	
 	for (f=qm->first_frag, i=0; (char*)f<(char*)qm->last_frag;
 			f=FRAG_NEXT(f), i++){
-		if (f->u.nxt_free==0){
-			/* it might be in-use or the last free fragm. in a free list 
-			   => search the free frags of the same size for a possible
-			   match --andrei*/
-			hash=GET_HASH(f->size);
-			for(free_frag=qm->free_hash[hash].first;
-					free_frag && (free_frag!=f);
-					free_frag=free_frag->u.nxt_free);
-			if (free_frag==0){ /* not found among the free frag */
-				x = get_mem_counter(&root,f);
-				x->count++;
-				x->size+=f->size;
-			}
+		if (!fm_is_free(f)){
+			x = get_mem_counter(&root,f);
+			x->count++;
+			x->size+=f->size;
 		}
 	}
 	x = root;
@@ -960,9 +942,294 @@ void fm_sums(struct fm_block* qm)
 	}
 	LOG_(DEFAULT_FACILITY, memlog, "fm_status: ",
 			"-----------------------------\n");
+
+}
+
+
+void fm_mod_get_stats(void *qmp, void **fm_rootp)
+{
+	if (!fm_rootp) {
+		return ;
+	}
+
+	LM_DBG("get fm memory statistics\n");
+
+	struct fm_block *qm = (struct fm_block *) qmp;
+	mem_counter **fm_root = (mem_counter **) fm_rootp;
+	struct fm_frag* f;
+	int i;
+	mem_counter *x;
+
+	if (!qm) return ;
+
+	/* update fragment detail list */
+	for (f=qm->first_frag, i=0; (char*)f<(char*)qm->last_frag;
+			f=FRAG_NEXT(f), i++){
+		if (f->is_free==0){
+			x = get_mem_counter(fm_root,f);
+			x->count++;
+			x->size+=f->size;
+		}
+	}
+
+    return ;
+}
+
+void fm_mod_free_stats(void *fm_rootp)
+{
+	if (!fm_rootp) {
+		return ;
+	}
+
+	LM_DBG("free fm memory statistics\n");
+
+	mem_counter *root = (mem_counter *) fm_rootp;
+	mem_counter *new, *old;
+	new = root;
+	old = root;
+
+	while (new) {
+		old = new;
+		new = new->next;
+		free(old);
+	}
+}
+#else
+void fm_sums(void *qmp)
+{
+	struct fm_block* qm;
+	int memlog;
+
+	qm = (struct fm_block*)qmp;
+	memlog=cfg_get(core, core_cfg, memlog);
+	LOG_(DEFAULT_FACILITY, memlog, "fm_sums: ", "not available (%p)\n", qm);
+	return;
+}
+
+void fm_mod_get_stats(void *qmp, void **fm_rootp)
+{
+	LM_WARN("Enable DBG_F_MALLOC for getting statistics\n");
+	return ;
+}
+
+void fm_mod_free_stats(void *fm_rootp)
+{
+	LM_WARN("Enable DBG_F_MALLOC for freeing statistics\n");
+	return ;
 }
 #endif /* DBG_F_MALLOC */
 
 
+/*memory manager core api*/
+static char *_fm_mem_name = "f_malloc";
+
+/* PKG - private memory API*/
+static char *_fm_pkg_pool = 0;
+static struct fm_block *_fm_pkg_block = 0;
+
+/**
+ * \brief Destroy memory pool
+ */
+void fm_malloc_destroy_pkg_manager(void)
+{
+	if (_fm_pkg_pool) {
+		free(_fm_pkg_pool);
+		_fm_pkg_pool = 0;
+	}
+	_fm_pkg_block = 0;
+}
+
+/**
+ * \brief Init memory pool
+ */
+int fm_malloc_init_pkg_manager(void)
+{
+	sr_pkg_api_t ma;
+	_fm_pkg_pool = malloc(pkg_mem_size);
+	if (_fm_pkg_pool)
+		_fm_pkg_block=fm_malloc_init(_fm_pkg_pool, pkg_mem_size, MEM_TYPE_PKG);
+	if (_fm_pkg_block==0){
+		LOG(L_CRIT, "could not initialize fm pkg memory pool\n");
+		fprintf(stderr, "Too much fm pkg memory demanded: %ld bytes\n",
+						pkg_mem_size);
+		return -1;
+	}
+
+	memset(&ma, 0, sizeof(sr_pkg_api_t));
+	ma.mname      = _fm_mem_name;
+	ma.mem_pool   = _fm_pkg_pool;
+	ma.mem_block  = _fm_pkg_block;
+	ma.xmalloc    = fm_malloc;
+	ma.xfree      = fm_free;
+	ma.xrealloc   = fm_realloc;
+	ma.xstatus    = fm_status;
+	ma.xinfo      = fm_info;
+	ma.xavailable = fm_available;
+	ma.xsums      = fm_sums;
+	ma.xdestroy   = fm_malloc_destroy_pkg_manager;
+	ma.xmodstats  = fm_mod_get_stats;
+	ma.xfmodstats = fm_mod_free_stats;
+
+	return pkg_init_api(&ma);
+}
+
+
+/* SHM - shared memory API*/
+static void *_fm_shm_pool = 0;
+static struct fm_block *_fm_shm_block = 0;
+
+/*SHM wrappers to sync the access to memory block*/
+#ifdef DBG_F_MALLOC
+void* fm_shm_malloc(void* qmp, unsigned long size,
+					const char* file, const char* func, unsigned int line, const char* mname)
+{
+	void *r;
+	shm_lock();
+	r = fm_malloc(qmp, size, file, func, line, mname);
+	shm_unlock();
+	return r;
+}
+void* fm_shm_realloc(void* qmp, void* p, unsigned long size,
+					const char* file, const char* func, unsigned int line, const char* mname)
+{
+	void *r;
+	shm_lock();
+	r = fm_realloc(qmp, p, size, file, func, line, mname);
+	shm_unlock();
+	return r;
+}
+void* fm_shm_resize(void* qmp, void* p, unsigned long size,
+					const char* file, const char* func, unsigned int line, const char* mname)
+{
+	void *r;
+	shm_lock();
+	if(p) fm_free(qmp, p, file, func, line, mname);
+	r = fm_malloc(qmp, size, file, func, line, mname);
+	shm_unlock();
+	return r;
+}
+void fm_shm_free(void* qmp, void* p, const char* file, const char* func,
+				unsigned int line, const char* mname)
+{
+	shm_lock();
+	fm_free(qmp, p, file, func, line, mname);
+	shm_unlock();
+}
+#else
+void* fm_shm_malloc(void* qmp, unsigned long size)
+{
+	void *r;
+	shm_lock();
+	r = fm_malloc(qmp, size);
+	shm_unlock();
+	return r;
+}
+void* fm_shm_realloc(void* qmp, void* p, unsigned long size)
+{
+	void *r;
+	shm_lock();
+	r = fm_realloc(qmp, p, size);
+	shm_unlock();
+	return r;
+}
+void* fm_shm_resize(void* qmp, void* p, unsigned long size)
+{
+	void *r;
+	shm_lock();
+	if(p) fm_free(qmp, p);
+	r = fm_malloc(qmp, size);
+	shm_unlock();
+	return r;
+}
+void fm_shm_free(void* qmp, void* p)
+{
+	shm_lock();
+	fm_free(qmp, p);
+	shm_unlock();
+}
+#endif
+void fm_shm_status(void* qmp)
+{
+	shm_lock();
+	fm_status(qmp);
+	shm_unlock();
+}
+void fm_shm_info(void* qmp, struct mem_info* info)
+{
+	shm_lock();
+	fm_info(qmp, info);
+	shm_unlock();
+
+}
+unsigned long fm_shm_available(void* qmp)
+{
+	unsigned long r;
+	shm_lock();
+	r = fm_available(qmp);
+	shm_unlock();
+	return r;
+}
+void fm_shm_sums(void* qmp)
+{
+	shm_lock();
+	fm_sums(qmp);
+	shm_unlock();
+}
+
+
+/**
+ * \brief Destroy memory pool
+ */
+void fm_malloc_destroy_shm_manager(void)
+{
+	/*shm pool from core - nothing to do*/
+	_fm_shm_pool = 0;
+	_fm_shm_block = 0;
+}
+
+/**
+ * \brief Init memory pool
+ */
+int fm_malloc_init_shm_manager(void)
+{
+	sr_shm_api_t ma;
+	_fm_shm_pool = shm_core_get_pool();
+	if (_fm_shm_pool)
+		_fm_shm_block=fm_malloc_init(_fm_shm_pool, shm_mem_size, MEM_TYPE_SHM);
+	if (_fm_shm_block==0){
+		LOG(L_CRIT, "could not initialize fm shm memory pool\n");
+		fprintf(stderr, "Too much fm shm memory demanded: %ld bytes\n",
+						shm_mem_size);
+		return -1;
+	}
+
+	memset(&ma, 0, sizeof(sr_shm_api_t));
+	ma.mname          = _fm_mem_name;
+	ma.mem_pool       = _fm_shm_pool;
+	ma.mem_block      = _fm_shm_block;
+	ma.xmalloc        = fm_shm_malloc;
+	ma.xmalloc_unsafe = fm_malloc;
+	ma.xfree          = fm_shm_free;
+	ma.xfree_unsafe   = fm_free;
+	ma.xrealloc       = fm_shm_realloc;
+	ma.xresize        = fm_shm_resize;
+	ma.xstatus        = fm_shm_status;
+	ma.xinfo          = fm_shm_info;
+	ma.xavailable     = fm_shm_available;
+	ma.xsums          = fm_shm_sums;
+	ma.xdestroy       = fm_malloc_destroy_shm_manager;
+	ma.xmodstats      = fm_mod_get_stats;
+	ma.xfmodstats     = fm_mod_free_stats;
+
+	if(shm_init_api(&ma)<0) {
+		LM_ERR("cannot initialize the core shm api\n");
+		return -1;
+	}
+	if(shm_core_lock_init()<0) {
+		LM_ERR("cannot initialize the core shm lock\n");
+		return -1;
+	}
+	return 0;
+}
 
 #endif
