@@ -1,5 +1,4 @@
 /**
- * $Id$
  *
  * Copyright (C) 2008 Elena-Ramona Modroiu (asipto.com)
  *
@@ -252,11 +251,14 @@ ht_t* ht_get_table(str *name)
 	return NULL;
 }
 
-int ht_add_table(str *name, int autoexp, str *dbtable, int size, int dbmode,
-		int itype, int_str *ival, int updateexpire, int dmqreplicate)
+int ht_add_table(str *name, int autoexp, str *dbtable, str *dbcols, int size,
+		int dbmode, int itype, int_str *ival, int updateexpire,
+		int dmqreplicate)
 {
 	unsigned int htid;
 	ht_t *ht;
+	int c;
+	int i;
 
 	htid = ht_compute_hash(name);
 
@@ -297,6 +299,48 @@ int ht_add_table(str *name, int autoexp, str *dbtable, int size, int dbmode,
 	if(ival!=NULL)
 		ht->initval = *ival;
 	ht->dmqreplicate = dmqreplicate;
+
+	if(dbcols!=NULL && dbcols->s!=NULL && dbcols->len>0) {
+		ht->scols[0].s = (char*)shm_malloc((1+dbcols->len)*sizeof(char));
+		if(ht->scols[0].s==NULL) {
+			LM_ERR("no more shm memory\n");
+			shm_free(ht);
+			return -1;
+		}
+		memset(ht->scols[0].s, 0, (1+dbcols->len)*sizeof(char));
+		memcpy(ht->scols[0].s, dbcols->s, dbcols->len);
+		ht->scols[0].len = dbcols->len;
+		c = 0;
+		for(i=0; i<dbcols->len; i++) {
+			if(ht->scols[0].s[i]==',') {
+				ht->scols[c].len = (ht->scols[0].s + i - ht->scols[c].s);
+				LM_DBG("db table column[%d]='%.*s'\n", c,
+						ht->scols[c].len, ht->scols[c].s);
+				c++;
+				if(c>=HT_MAX_COLS) {
+					LM_ERR("too many columns %d\n", c);
+					shm_free(ht->scols[0].s);
+					shm_free(ht);
+					return -1;
+				}
+				ht->scols[c].s = ht->scols[0].s + i + 1;
+				ht->scols[c].len = ht->scols[0].s + dbcols->len - ht->scols[c].s;
+			}
+		}
+		LM_DBG("db table column[%d]='%.*s'\n", c,
+				ht->scols[c].len, ht->scols[c].s);
+		if(c==0) {
+			LM_ERR("there must be at least two columns (prefix, value)\n");
+			shm_free(ht->scols[0].s);
+			shm_free(ht);
+			return -1;
+		}
+		ht->ncols = c + 1;
+		ht->pack[0] = 'l';
+		ht->pack[1] = ',';
+		ht->pack[2] = '*';
+	}
+
 	ht->next = _ht_root;
 	_ht_root = ht;
 	return 0;
@@ -347,7 +391,7 @@ int ht_init_tables(void)
 		{
 			if(lock_init(&ht->entries[i].lock)==0)
 			{
-				LM_ERR("cannot initalize lock[%d] in [%.*s]\n", i,
+				LM_ERR("cannot initialize lock[%d] in [%.*s]\n", i,
 						ht->name.len, ht->name.s);
 				i--;
 				while(i>=0)
@@ -461,7 +505,10 @@ int ht_set_cell(ht_t *ht, str *name, int type, int_str *val, int mode)
 						}
 						cell->next = it->next;
 						cell->prev = it->prev;
-						cell->expire = now + ht->htexpire;
+						if(ht->updateexpire)
+							cell->expire = now + ht->htexpire;
+						else
+							cell->expire = it->expire;
 						if(it->prev)
 							it->prev->next = cell;
 						else
@@ -490,7 +537,10 @@ int ht_set_cell(ht_t *ht, str *name, int type, int_str *val, int mode)
 						if(mode) ht_slot_unlock(ht, idx);
 						return -1;
 					}
-					cell->expire = now + ht->htexpire;
+					if(ht->updateexpire)
+						cell->expire = now + ht->htexpire;
+					else
+						cell->expire = it->expire;
 					cell->next = it->next;
 					cell->prev = it->prev;
 					if(it->prev)
@@ -650,7 +700,8 @@ ht_cell_t* ht_cell_value_add(ht_t *ht, str *name, int val, int mode,
 				return NULL;
 			} else {
 				it->value.n += val;
-				it->expire = now + ht->htexpire;
+				if(ht->updateexpire)
+					it->expire = now + ht->htexpire;
 				if(old!=NULL)
 				{
 					if(old->msize>=it->msize)
@@ -673,7 +724,10 @@ ht_cell_t* ht_cell_value_add(ht_t *ht, str *name, int val, int mode,
 	}
 	/* add val if htable has an integer init value */
 	if(ht->flags!=PV_VAL_INT)
+	{
+		if(mode) ht_slot_unlock(ht, idx);
 		return NULL;
+	}
 	isval.n = ht->initval.n + val;
 	it = ht_cell_new(name, 0, &isval, hid);
 	if(it == NULL)
@@ -819,6 +873,7 @@ int ht_table_spec(char *spec)
 	keyvalue_t kval;
 	str name;
 	str dbtable = {0, 0};
+	str dbcols  = {0, 0};
 	unsigned int autoexpire = 0;
 	unsigned int size = 4;
 	unsigned int dbmode = 0;
@@ -854,6 +909,10 @@ int ht_table_spec(char *spec)
 			dbtable = tok;
 			LM_DBG("htable [%.*s] - dbtable [%.*s]\n", name.len, name.s,
 					dbtable.len, dbtable.s);
+		} else if(pit->name.len==4 && strncmp(pit->name.s, "cols", 4)==0) {
+			dbcols = tok;
+			LM_DBG("htable [%.*s] - dbcols [%.*s]\n", name.len, name.s,
+					dbcols.len, dbcols.s);
 		} else if(pit->name.len==10 && strncmp(pit->name.s, "autoexpire", 10)==0) {
 			if(str2int(&tok, &autoexpire)!=0)
 				goto error;
@@ -888,7 +947,7 @@ int ht_table_spec(char *spec)
 		} else { goto error; }
 	}
 
-	return ht_add_table(&name, autoexpire, &dbtable, size, dbmode,
+	return ht_add_table(&name, autoexpire, &dbtable, &dbcols, size, dbmode,
 			itype, &ival, updateexpire, dmqreplicate);
 
 error:
@@ -923,7 +982,7 @@ int ht_db_sync_tables(void)
 	ht = _ht_root;
 	while(ht)
 	{
-		if(ht->dbtable.len>0 && ht->dbmode!=0)
+		if(ht->dbtable.len>0 && ht->dbmode!=0 && ht->ncols==0)
 		{
 			LM_DBG("sync db table [%.*s] from ht [%.*s]\n",
 					ht->dbtable.len, ht->dbtable.s,
@@ -955,6 +1014,8 @@ int ht_has_autoexpire(void)
 	return 0;
 }
 
+extern int ht_timer_procs;
+
 void ht_timer(unsigned int ticks, void *param)
 {
 	ht_t *ht;
@@ -962,18 +1023,24 @@ void ht_timer(unsigned int ticks, void *param)
 	ht_cell_t *it0;
 	time_t now;
 	int i;
+	int istart;
+	int istep;
 
 	if(_ht_root==NULL)
 		return;
 
 	now = time(NULL);
-	
+
+	istart = (int)(long)param;
+	if(ht_timer_procs<=0) istep = 1;
+	else istep = ht_timer_procs;
+
 	ht = _ht_root;
 	while(ht)
 	{
 		if(ht->htexpire>0)
 		{
-			for(i=0; i<ht->htsize; i++)
+			for(i=istart; i<ht->htsize; i+=istep)
 			{
 				/* free entries */
 				ht_slot_lock(ht, i);
