@@ -13,19 +13,16 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License 
- * along with this program; if not, write to the Free Software 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
- * History:
- * --------
- *  2006-08-15  initial version (Anca Vamanu)
  */
 
 /*!
  * \file
  * \brief Kamailio presence module :: Presentity handling
- * \ingroup presence 
+ * \ingroup presence
  */
 
 
@@ -38,17 +35,20 @@
 #include "../../hashes.h"
 #include "../../dprint.h"
 #include "../../mem/shm_mem.h"
+#include "../../xavp.h"
 #include "../../str.h"
-#include "../alias_db/alias_db.h"
 #include "../../data_lump_rpl.h"
 #include "presentity.h"
-#include "presence.h" 
+#include "presence.h"
 #include "notify.h"
 #include "publish.h"
 #include "hash.h"
 #include "utils_func.h"
 
 
+/* base priority value (20150101T000000) */
+#define PRES_PRIORITY_TBASE	1420070400
+
 xmlNodePtr xmlNodeGetNodeByName(xmlNodePtr node, const char *name,
 													const char *ns);
 static str pu_200_rpl  = str_init("OK");
@@ -101,9 +101,9 @@ int publ_send200ok(struct sip_msg *msg, int lexpire, str etag)
 	if (msg == NULL)
 		return 0;
 
-	LM_DBG("send 200OK reply\n");	
+	LM_DBG("send 200OK reply\n");
 	LM_DBG("etag= %s - len= %d\n", etag.s, etag.len);
-	
+
 	hdr_append.s = buf;
 	hdr_append.s[0]='\0';
 	hdr_append.len = snprintf(hdr_append.s, buf_len, "Expires: %d\r\n",
@@ -119,7 +119,7 @@ int publ_send200ok(struct sip_msg *msg, int lexpire, str etag)
 		goto error;
 	}
 	hdr_append.s[hdr_append.len]= '\0';
-		
+
 	if (add_lump_rpl( msg, hdr_append.s, hdr_append.len, LUMP_RPL_HDR)==0 )
 	{
 		LM_ERR("unable to add lump_rl\n");
@@ -167,18 +167,42 @@ error:
 		pkg_free(hdr_append2.s);
 
 	return -1;
-}	
-presentity_t* new_presentity( str* domain,str* user,int expires, 
+}
+
+/**
+ * get priority value for presence document
+ */
+unsigned int pres_get_priority(void)
+{
+	sr_xavp_t *vavp = NULL;
+	str vname = str_init("priority");
+
+	if(pres_xavp_cfg.s==NULL || pres_xavp_cfg.len<=0) {
+		return 0;
+	}
+
+	vavp = xavp_get_child_with_ival(&pres_xavp_cfg, &vname);
+	if(vavp!=NULL) {
+		return (unsigned int)vavp->val.v.i;
+	}
+
+	return (unsigned int)(time(NULL) - PRES_PRIORITY_TBASE);
+}
+
+/**
+ * create new presentity record
+ */
+presentity_t* new_presentity( str* domain,str* user,int expires,
 		pres_ev_t* event, str* etag, str* sender)
 {
 	presentity_t *presentity= NULL;
 	int size, init_len;
-	
+
 	/* allocating memory for presentity */
 	size = sizeof(presentity_t)+ domain->len+ user->len+ etag->len +1;
 	if(sender)
 		size+= sizeof(str)+ sender->len* sizeof(char);
-	
+
 	init_len= size;
 
 	presentity = (presentity_t*)pkg_malloc(size);
@@ -192,8 +216,8 @@ presentity_t* new_presentity( str* domain,str* user,int expires,
 	presentity->domain.s = (char*)presentity+ size;
 	strncpy(presentity->domain.s, domain->s, domain->len);
 	presentity->domain.len = domain->len;
-	size+= domain->len;	
-	
+	size+= domain->len;
+
 	presentity->user.s = (char*)presentity+size;
 	strncpy(presentity->user.s, user->s, user->len);
 	presentity->user.len = user->len;
@@ -205,7 +229,7 @@ presentity_t* new_presentity( str* domain,str* user,int expires,
 	presentity->etag.len = etag->len;
 
 	size+= etag->len+1;
-	
+
 	if(sender)
 	{
 		presentity->sender= (str*)((char*)presentity+ size);
@@ -224,8 +248,9 @@ presentity_t* new_presentity( str* domain,str* user,int expires,
 	presentity->event= event;
 	presentity->expires = expires;
 	presentity->received_time= (int)time(NULL);
+	presentity->priority = pres_get_priority();
 	return presentity;
-    
+
 error:
 	if(presentity)
 		pkg_free(presentity);
@@ -271,9 +296,9 @@ int check_if_dialog(str body, int *is_dialog)
 int update_presentity(struct sip_msg* msg, presentity_t* presentity, str* body,
 		int new_t, int* sent_reply, char* sphere)
 {
-	db_key_t query_cols[12], update_keys[8], result_cols[5];
-	db_op_t  query_ops[12];
-	db_val_t query_vals[12], update_vals[8];
+	db_key_t query_cols[13], update_keys[9], result_cols[6];
+	db_op_t  query_ops[13];
+	db_val_t query_vals[13], update_vals[9];
 	db1_res_t *result= NULL;
 	int n_query_cols = 0;
 	int n_update_cols = 0;
@@ -303,7 +328,7 @@ int update_presentity(struct sip_msg* msg, presentity_t* presentity, str* body,
 			goto error;
 		}
 	}
-	
+
 	if(uandd_to_uri(presentity->user, presentity->domain, &pres_uri)< 0)
 	{
 		LM_ERR("constructing uri from user and domain\n");
@@ -317,7 +342,7 @@ int update_presentity(struct sip_msg* msg, presentity_t* presentity, str* body,
 	query_vals[n_query_cols].nul = 0;
 	query_vals[n_query_cols].val.str_val = presentity->domain;
 	n_query_cols++;
-	
+
 	query_cols[n_query_cols] = &str_username_col;
 	query_ops[n_query_cols] = OP_EQ;
 	query_vals[n_query_cols].type = DB1_STR;
@@ -342,7 +367,7 @@ int update_presentity(struct sip_msg* msg, presentity_t* presentity, str* body,
 	result_cols[rez_body_col= n_result_cols++] = &str_body_col;
 	result_cols[rez_sender_col= n_result_cols++] = &str_sender_col;
 
-	if(new_t) 
+	if(new_t)
 	{
 		/* insert new record in hash_table */
 
@@ -352,8 +377,8 @@ int update_presentity(struct sip_msg* msg, presentity_t* presentity, str* body,
 			LM_ERR("inserting record in hash table\n");
 			goto error;
 		}
-		
-		/* insert new record into database */	
+
+		/* insert new record into database */
 		query_cols[n_query_cols] = &str_sender_col;
 		query_vals[n_query_cols].type = DB1_STR;
 		query_vals[n_query_cols].nul = 0;
@@ -372,13 +397,19 @@ int update_presentity(struct sip_msg* msg, presentity_t* presentity, str* body,
 		query_vals[n_query_cols].nul = 0;
 		query_vals[n_query_cols].val.str_val = *body;
 		n_query_cols++;
-		
+
 		query_cols[n_query_cols] = &str_received_time_col;
 		query_vals[n_query_cols].type = DB1_INT;
 		query_vals[n_query_cols].nul = 0;
 		query_vals[n_query_cols].val.int_val = presentity->received_time;
 		n_query_cols++;
-		
+
+		query_cols[n_query_cols] = &str_priority_col;
+		query_vals[n_query_cols].type = DB1_INT;
+		query_vals[n_query_cols].nul = 0;
+		query_vals[n_query_cols].val.int_val = presentity->priority;
+		n_query_cols++;
+
 		if (presentity->expires != -1)
 		{
 			/* A real PUBLISH */
@@ -388,16 +419,25 @@ int update_presentity(struct sip_msg* msg, presentity_t* presentity, str* body,
 			query_vals[n_query_cols].val.int_val = presentity->expires+
 								(int)time(NULL);
 			n_query_cols++;
-	
-			if (pa_dbf.use_table(pa_db, &presentity_table) < 0) 
+
+			if (pa_dbf.use_table(pa_db, &presentity_table) < 0)
 			{
 				LM_ERR("unsuccessful use_table\n");
 				goto error;
 			}
 
+			if (pa_dbf.start_transaction)
+			{
+				if (pa_dbf.start_transaction(pa_db, db_table_lock) < 0)
+				{
+					LM_ERR("in start_transaction\n");
+					goto error;
+				}
+			}
+
 			LM_DBG("inserting %d cols into table\n",n_query_cols);
-				
-			if (pa_dbf.insert(pa_db, query_cols, query_vals, n_query_cols) < 0) 
+
+			if (pa_dbf.insert(pa_db, query_cols, query_vals, n_query_cols) < 0)
 			{
 				LM_ERR("inserting new record in database\n");
 				goto error;
@@ -411,8 +451,8 @@ int update_presentity(struct sip_msg* msg, presentity_t* presentity, str* body,
 			query_vals[n_query_cols].nul = 0;
 			query_vals[n_query_cols].val.int_val = -1;
 			n_query_cols++;
-	
-			if (pa_dbf.use_table(pa_db, &presentity_table) < 0) 
+
+			if (pa_dbf.use_table(pa_db, &presentity_table) < 0)
 			{
 				LM_ERR("unsuccessful use_table\n");
 				goto error;
@@ -434,25 +474,11 @@ int update_presentity(struct sip_msg* msg, presentity_t* presentity, str* body,
 			}
 
 
-			if (pa_dbf.replace(pa_db, query_cols, query_vals, n_query_cols, 4, 0) < 0) 
+			if (pa_dbf.replace(pa_db, query_cols, query_vals, n_query_cols, 4, 0) < 0)
 			{
 				LM_ERR("replacing record in database\n");
-				if (pa_dbf.abort_transaction)
-				{
-					if (pa_dbf.abort_transaction(pa_db) < 0)
-						LM_ERR("in abort_transaction\n");
-				}
 				goto error;
 			}
-
-			if (pa_dbf.end_transaction)
-			{
-				if (pa_dbf.end_transaction(pa_db) < 0)
-				{
-					LM_ERR("in end_transaction\n");
-					goto error;
-				}
-			}
 		}
 
 		if( publ_send200ok(msg, presentity->expires, presentity->etag)< 0)
@@ -464,19 +490,28 @@ int update_presentity(struct sip_msg* msg, presentity_t* presentity, str* body,
 		goto send_notify;
 	}
 	else
-	{	
+	{
 
-		if (pa_dbf.use_table(pa_db, &presentity_table) < 0) 
+		if (pa_dbf.use_table(pa_db, &presentity_table) < 0)
 		{
 			LM_ERR("unsuccessful sql use table\n");
 			goto error;
 		}
 
+		if (pa_dbf.start_transaction)
+		{
+			if (pa_dbf.start_transaction(pa_db, db_table_lock) < 0)
+			{
+				LM_ERR("in start_transaction\n");
+				goto error;
+			}
+		}
+
 		if(EVENT_DIALOG_SLA(presentity->event->evp))
 		{
 
 			if (pa_dbf.query (pa_db, query_cols, query_ops, query_vals,
-			 result_cols, n_query_cols, n_result_cols, 0, &result) < 0) 
+					result_cols, n_query_cols, n_result_cols, 0, &result) < 0)
 			{
 				LM_ERR("unsuccessful sql query\n");
 				goto error;
@@ -486,7 +521,7 @@ int update_presentity(struct sip_msg* msg, presentity_t* presentity, str* body,
 
 			if (!(result->n > 0))
 				goto send_412;
-			
+
 			db_record_exists= 1;
 			/* analize if previous body has a dialog */
 			row = &result->rows[0];
@@ -518,23 +553,23 @@ int update_presentity(struct sip_msg* msg, presentity_t* presentity, str* body,
 			LM_DBG("old_sender = %.*s\n", sender.len, sender.s );
 			if(presentity->sender)
 			{
-				if(!(presentity->sender->len == sender.len && 
-				strncmp(presentity->sender->s, sender.s, sender.len)== 0))
-					 bla_update_publish= 0;
+				if(!(presentity->sender->len == sender.len &&
+						presence_sip_uri_match(presentity->sender, &sender)== 0))
+					bla_update_publish= 0;
 			}
 after_dialog_check:
 			pa_dbf.free_result(pa_db, result);
 			result = NULL;
-			
+
 		}
 
-		if(presentity->expires <= 0) 
+		if(presentity->expires <= 0)
 		{
 
 			if (!db_record_exists)
 			{
 				if (pa_dbf.query (pa_db, query_cols, query_ops, query_vals,
-					result_cols, n_query_cols, n_result_cols, 0, &result) < 0) 
+					result_cols, n_query_cols, n_result_cols, 0, &result) < 0)
 				{
 					LM_ERR("unsuccessful sql query\n");
 					goto error;
@@ -623,7 +658,7 @@ after_dialog_check:
 		}
 
 		if(presentity->event->etag_not_new== 0)
-		{	
+		{
 			/* generate another etag */
 			unsigned int publ_nr;
 			str str_publ_nr= {0, 0};
@@ -638,7 +673,7 @@ after_dialog_check:
 			{
 				LM_ERR("wrong etag\n");
 				goto error;
-			}	
+			}
 			str_publ_nr.s= dot+1;
 			str_publ_nr.len--;
 
@@ -654,7 +689,7 @@ after_dialog_check:
 				goto error;
 			}
 			etag.len=(strlen(etag.s));
-			
+
 			cur_etag= etag;
 
 			update_keys[n_update_cols] = &str_etag_col;
@@ -666,7 +701,7 @@ after_dialog_check:
 		}
 		else
 			cur_etag= presentity->etag;
-			
+
 		update_keys[n_update_cols] = &str_expires_col;
 		update_vals[n_update_cols].type = DB1_INT;
 		update_vals[n_update_cols].nul = 0;
@@ -680,6 +715,12 @@ after_dialog_check:
 		update_vals[n_update_cols].val.int_val= presentity->received_time;
 		n_update_cols++;
 
+		update_keys[n_update_cols] = &str_priority_col;
+		update_vals[n_update_cols].type = DB1_INT;
+		update_vals[n_update_cols].nul = 0;
+		update_vals[n_update_cols].val.int_val= presentity->priority;
+		n_update_cols++;
+
 		if(body && body->s)
 		{
 			update_keys[n_update_cols] = &str_body_col;
@@ -689,7 +730,7 @@ after_dialog_check:
 			n_update_cols++;
 
 			/* updated stored sphere */
-			if(sphere_enable && 
+			if(sphere_enable &&
 					presentity->event->evp->type== EVENT_PRESENCE)
 			{
 				if( publ_cache_enabled &&
@@ -700,8 +741,7 @@ after_dialog_check:
 				}
 			}
 		}
-		
-		
+
 		if( presentity->sender)
 		{
 			update_keys[n_update_cols] = &str_sender_col;
@@ -715,14 +755,14 @@ after_dialog_check:
 		if (!pa_dbf.affected_rows && !db_record_exists)
 		{
 			if (pa_dbf.query (pa_db, query_cols, query_ops, query_vals,
-			 result_cols, n_query_cols, n_result_cols, 0, &result) < 0) 
+					result_cols, n_query_cols, n_result_cols, 0, &result) < 0)
 			{
 				LM_ERR("unsuccessful sql query\n");
 				goto error;
 			}
 			if(result== NULL)
 				goto error;
-			
+
 			if (!(result->n > 0))
 				goto send_412;
 
@@ -732,7 +772,7 @@ after_dialog_check:
 		}
 
 		if( pa_dbf.update( pa_db,query_cols, query_ops, query_vals,
-				update_keys, update_vals, n_query_cols, n_update_cols )<0) 
+				update_keys, update_vals, n_query_cols, n_update_cols )<0)
 		{
 			LM_ERR("updating published info in database\n");
 			goto error;
@@ -800,6 +840,15 @@ done:
 	if(pres_uri.s)
 		pkg_free(pres_uri.s);
 
+	if (pa_dbf.end_transaction)
+	{
+		if (pa_dbf.end_transaction(pa_db) < 0)
+		{
+			LM_ERR("in end_transaction\n");
+			goto error;
+		}
+	}
+
 	return 0;
 
 send_412:
@@ -821,21 +870,28 @@ error:
 		pa_dbf.free_result(pa_db, result);
 	if(etag.s)
 		pkg_free(etag.s);
-	if(rules_doc)
-	{
+	if(rules_doc) {
 		if(rules_doc->s)
 			pkg_free(rules_doc->s);
 		pkg_free(rules_doc);
 	}
-	if(pres_uri.s)
+	if(pres_uri.s) {
 		pkg_free(pres_uri.s);
+		pres_uri.s = NULL;
+	}
+
+	if (pa_dbf.abort_transaction) {
+		if (pa_dbf.abort_transaction(pa_db) < 0) {
+			LM_ERR("in abort_transaction\n");
+		}
+	}
 
 	return ret;
 }
 
 int pres_htable_restore(void)
 {
-	/* query all records from presentity table and insert records 
+	/* query all records from presentity table and insert records
 	 * in presentity table */
 	db_key_t result_cols[6];
 	db1_res_t *result= NULL;
@@ -887,7 +943,7 @@ int pres_htable_restore(void)
 
 			if(row_vals[expires_col].val.int_val< (int)time(NULL))
 				continue;
-		
+
 			sphere= NULL;
 			user.s= (char*)row_vals[user_col].val.string_val;
 			user.len= strlen(user.s);
@@ -911,7 +967,7 @@ int pres_htable_restore(void)
 				goto error;
 			}
 			/* insert in hash_table*/
-	
+
 			if(sphere_enable && event== EVENT_PRESENCE )
 			{
 				body.s= (char*)row_vals[body_col].val.string_val;
@@ -941,7 +997,7 @@ int pres_htable_restore(void)
 error:
 	if(result)
 		pa_dbf.free_result(pa_db, result);
-	return -1;	
+	return -1;
 }
 
 char* extract_sphere(str body)
@@ -951,7 +1007,7 @@ char* extract_sphere(str body)
 	xmlDocPtr doc= NULL;
 	xmlNodePtr node;
 	char* cont, *sphere= NULL;
-	
+
 
 	doc= xmlParseMemory(body.s, body.len);
 	if(doc== NULL)
@@ -961,7 +1017,7 @@ char* extract_sphere(str body)
 	}
 
 	node= xmlNodeGetNodeByName(doc->children, "sphere", "rpid");
-	
+
 	if(node== NULL)
 		node= xmlNodeGetNodeByName(doc->children, "sphere", "r");
 
@@ -1035,7 +1091,7 @@ char* get_sphere(str* pres_uri)
 	if ( publ_cache_enabled )
 	{
 		/* search in hash table*/
-		hash_code= core_hash(pres_uri, NULL, phtable_size);
+		hash_code= core_case_hash(pres_uri, NULL, phtable_size);
 
 		lock_get(&pres_htable[hash_code].lock);
 
@@ -1085,23 +1141,27 @@ char* get_sphere(str* pres_uri)
 	n_query_cols++;
 
 	result_cols[n_result_cols++] = &str_body_col;
-	
-	if (pa_dbf.use_table(pa_db, &presentity_table) < 0) 
+
+	if (pa_dbf.use_table(pa_db, &presentity_table) < 0)
 	{
 		LM_ERR("in use_table\n");
 		return NULL;
 	}
 
-	query_str = str_received_time_col;
+	if(pres_retrieve_order==1) {
+		query_str = str_priority_col;
+	} else {
+		query_str = str_received_time_col;
+	}
 	if (pa_dbf.query (pa_db, query_cols, 0, query_vals,
-		 result_cols, n_query_cols, n_result_cols, &query_str ,  &result) < 0) 
+			result_cols, n_query_cols, n_result_cols, &query_str ,  &result) < 0)
 	{
 		LM_ERR("failed to query %.*s table\n", presentity_table.len, presentity_table.s);
 		if(result)
 			pa_dbf.free_result(pa_db, result);
 		return NULL;
 	}
-	
+
 	if(result== NULL)
 		return NULL;
 
@@ -1127,7 +1187,7 @@ char* get_sphere(str* pres_uri)
 		LM_ERR("Empty notify body record\n");
 		goto error;
 	}
-	
+
 	sphere= extract_sphere(body);
 
 	pa_dbf.free_result(pa_db, result);
@@ -1227,7 +1287,7 @@ int mark_presentity_for_delete(presentity_t *pres)
 	if (RES_ROW_N(result) > 1)
 	{
 		/* More that one is prevented by DB constraint  - but handle
- 		   it anyway */
+		 * it anyway */
 		LM_ERR("Found %d presentities - expected 1\n", RES_ROW_N(result));
 
 		if (delete_presentity(pres) < 0)
@@ -1237,7 +1297,7 @@ int mark_presentity_for_delete(presentity_t *pres)
 		}
 
 		/* Want the calling function to continue properly so do not
-		   return an error */
+		 * return an error */
 		goto done;
 	}
 
@@ -1278,7 +1338,7 @@ int mark_presentity_for_delete(presentity_t *pres)
 	n_update_cols++;
 
 	if (pa_dbf.update(pa_db, query_cols, 0, query_vals, update_cols,
-			 update_vals, n_query_cols, n_update_cols) < 0)
+			update_vals, n_query_cols, n_update_cols) < 0)
 	{
 		LM_ERR("unsuccessful sql update operation");
 		goto error;
