@@ -1,5 +1,4 @@
-/*$Id$
- *
+/*
  * Copyright (C) 2001-2003 FhG Fokus
  *
  * This file is part of Kamailio, a free SIP server.
@@ -19,35 +18,21 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  *
- * History:
- * -------
- *  2003-02-28  scratchpad compatibility abandoned (jiri)
- *  2003-01-29: - rewriting actions (replace, search_append) now begin
- *                at the second line -- previously, they could affect
- *                first line too, which resulted in wrong calculation of
- *                forwarded requests and an error consequently
- *              - replace_all introduced
- *  2003-01-28  scratchpad removed (jiri)
- *  2003-01-18  append_urihf introduced (jiri)
- *  2003-03-10  module export interface updated to the new format (andrei)
- *  2003-03-16  flags export parameter added (janakj)
- *  2003-03-19  replaced all mallocs/frees w/ pkg_malloc/pkg_free (andrei)
- *  2003-04-97  actions permitted to be used from failure/reply routes (jiri)
- *  2003-04-21  remove_hf and is_present_hf introduced (jiri)
- *  2003-08-19  subst added (support for sed like res:s/re/repl/flags) (andrei)
- *  2003-08-20  subst_uri added (like above for uris) (andrei)
- *  2003-09-11  updated to new build_lump_rpl() interface (bogdan)
- *  2003-11-11: build_lump_rpl() removed, add_lump_rpl() has flags (bogdan)
- *  2004-05-09: append_time introduced (jiri)
- *  2004-07-06  subst_user added (like subst_uri but only for user) (sobomax)
- *  2004-11-12  subst_user changes (old serdev mails) (andrei)
- *  2005-07-05  is_method("name") to check method using id (ramona)
- *  2006-03-17  applied patch from Marc Haisenko <haisenko@comdasys.com> 
- *              for adding has_body() function (bogdan)
- *  2008-07-14  Moved some function declarations to a separate file (Ardjan Zwartjes) 
- *
  */
 
+/*!
+ * \file
+ * \brief Module interface
+ * \ingroup textops
+ * Module: \ref textops
+ */
+
+/**
+ * @defgroup Textops Various text operatoins on messages
+ * @brief Kamailio textops module
+ */
+
+
 
 #include "../../action.h"
 #include "../../sr_module.h"
@@ -58,6 +43,7 @@
 #include "../../mem/mem.h"
 #include "../../str.h"
 #include "../../re.h"
+#include "../../lvalue.h"
 #include "../../mod_fix.h"
 #include "../../parser/parse_uri.h"
 #include "../../parser/parse_hname2.h"
@@ -132,6 +118,9 @@ static int append_multibody_2(struct sip_msg* msg, char*, char *);
 static int append_multibody_3(struct sip_msg* msg, char*, char *, char *);
 static int fixup_multibody_f(void** param, int param_no);
 static int remove_multibody_f(struct sip_msg *msg, char *);
+static int get_body_part_raw_f(sip_msg_t* msg, char* ctype, char* ovar);
+static int get_body_part_f(sip_msg_t* msg, char* ctype, char* ovar);
+static int fixup_get_body_part(void** param, int param_no);
 static int is_method_f(struct sip_msg* msg, char* , char *);
 static int has_body_f(struct sip_msg *msg, char *type, char *str2 );
 static int in_list_f(struct sip_msg* _msg, char* _subject, char* _list,
@@ -305,6 +294,12 @@ static cmd_export_t cmds[]={
 	{"remove_body_part",     (cmd_function)remove_multibody_f,    1,
 		fixup_spve_null, 0,
 		REQUEST_ROUTE|FAILURE_ROUTE|BRANCH_ROUTE },
+	{"get_body_part_raw",    (cmd_function)get_body_part_raw_f,   2,
+		fixup_get_body_part, 0,
+		REQUEST_ROUTE|FAILURE_ROUTE|BRANCH_ROUTE|ONREPLY_ROUTE },
+	{"get_body_part",        (cmd_function)get_body_part_f,       2,
+		fixup_get_body_part, 0,
+		REQUEST_ROUTE|FAILURE_ROUTE|BRANCH_ROUTE|ONREPLY_ROUTE },
 
 	{0,0,0,0,0,0}
 };
@@ -2030,6 +2025,151 @@ static int remove_multibody_f(struct sip_msg* msg, char* p1)
 	return -1;
 }
 
+/**
+ *
+ */
+static int get_body_part_helper(sip_msg_t* msg, char* ctype, char* ovar, int mode)
+{
+	char *start, *end, *bstart;
+	char *body_headers_end; 
+	unsigned int len, t;
+	str content_type, body;
+	str boundary = {0,0};
+	pv_spec_t *dst;
+	pv_value_t val;
+
+	if(ctype==0) {
+		LM_ERR("invalid parameters\n");
+		return -1;
+	}
+
+	if(fixup_get_svalue(msg, (gparam_t*)ctype, &content_type)!=0) {
+		LM_ERR("unable to get content type\n");
+		return -1;
+	}
+
+	body.s = get_body(msg);
+	if (body.s == 0) {
+		LM_ERR("failed to get the message body\n");
+		return -1;
+	}
+	body.len = msg->len - (int)(body.s - msg->buf);
+	if (body.len == 0) {
+		LM_DBG("message body has zero length\n");
+		return -1;
+	}
+
+	if(get_boundary(msg, &boundary)!=0) {
+		LM_ERR("Cannot get boundary. Is body multipart?\n");
+		return -1;
+	}
+
+	start = body.s;
+	len = body.len;
+
+	/* note: header body can follow just after name: - fixit */
+	while (find_line_start("Content-Type: ", 14, &start, &len))
+	{
+		end = start + 14;
+		len = len - 14;
+		if (len > (content_type.len + 2)) {
+			if (strncasecmp(end, content_type.s, content_type.len)== 0)
+			{
+				LM_DBG("found content type %.*s\n",
+					content_type.len, content_type.s);
+				end = end + content_type.len;
+				if ((*end != 13) || (*(end + 1) != 10))
+				{
+					LM_ERR("no CRLF found after content type\n");
+					goto err;
+				}
+				end = end + 2;
+				len = len - content_type.len - 2;
+				body_headers_end = end;
+				if (find_line_start(boundary.s, boundary.len, &end,
+					&len))
+				{
+					LM_DBG("found boundary %.*s\n", boundary.len, boundary.s);
+					bstart = end;
+					end = end + boundary.len;
+					len = len - boundary.len;
+					if (!(t = get_line(end, len))) {
+						LM_ERR("no CRLF found after boundary\n");
+						goto err;
+					}
+					end += t; len = end-start;
+					pkg_free(boundary.s);
+					boundary.s = NULL;
+					if(mode==1) {
+						end = body_headers_end;
+						val.rs.s = end;
+						val.rs.len = bstart - val.rs.s;
+					} else {
+						val.rs.s = start;
+						val.rs.len = len;
+					}
+					LM_DBG("output result: %.*s\n", val.rs.len, val.rs.s);
+					val.flags = PV_VAL_STR;
+					dst = (pv_spec_t *)ovar;
+					dst->setf(msg, &dst->pvp, (int)EQ_T, &val);
+					return 1;
+				}
+				LM_ERR("boundary not found after content\n");
+				goto err;
+			}
+			start = end;
+		} else {
+			LM_ERR("failed to match on content-type\n");
+			goto err;
+		}
+	}
+ err:
+	if(boundary.s) pkg_free(boundary.s);
+	return -1;
+}
+
+/**
+ *
+ */
+static int get_body_part_raw_f(sip_msg_t* msg, char* ctype, char* ovar)
+{
+	return get_body_part_helper(msg, ctype, ovar, 0);
+}
+
+/**
+ *
+ */
+static int get_body_part_f(sip_msg_t* msg, char* ctype, char* ovar)
+{
+	return get_body_part_helper(msg, ctype, ovar, 1);
+}
+
+/*
+ * Fix get_body_part_raw params: content type (string that may contain pvars) and
+ * result (writable pvar).
+ */
+static int fixup_get_body_part(void** param, int param_no)
+{
+	if (param_no == 1) {
+		return fixup_spve_null(param, 1);
+	}
+
+	if (param_no == 2) {
+		if (fixup_pvar_null(param, 1) != 0) {
+			LM_ERR("failed to fixup result pvar\n");
+			return -1;
+		}
+		if (((pv_spec_t *)(*param))->setf == NULL) {
+		    LM_ERR("result pvar is not writeble\n");
+			return -1;
+		}
+		return 0;
+    }
+
+    LM_ERR("invalid parameter number <%d>\n", param_no);
+    return -1;
+}
+
 static int append_to_reply_f(struct sip_msg* msg, char* key, char* str0)
 {
 	str s0;
@@ -2126,6 +2266,11 @@ int add_hf_helper(struct sip_msg* msg, str *str1, str *str2,
 	len=s0.len;
 	if (str2) len+= str2->len + REQ_LINE(msg).uri.len;
 
+	if(len==0) {
+		LM_INFO("nothing to add\n");
+		return -1;
+	}
+
 	s  = (char*)pkg_malloc(len);
 	if (!s) {
 		LM_ERR("no pkg memory left\n");
@@ -2236,8 +2381,7 @@ static int hname_fixup(void** param, int param_no)
 	gp->v.str.s[gp->v.str.len] = ':';
 	gp->v.str.len++;
 	
-	if (parse_hname2(gp->v.str.s, gp->v.str.s
-				+ ((gp->v.str.len<4)?4:gp->v.str.len), &hdr)==0)
+	if (parse_hname2_short(gp->v.str.s, gp->v.str.s + gp->v.str.len, &hdr)==0)
 	{
 		LM_ERR("error parsing header name\n");
 		pkg_free(gp);
@@ -2779,7 +2923,7 @@ static int search_hf_f(struct sip_msg* msg, char* str_hf, char* re, char *flags)
 					return 1;
 			} else {
 				if(flags!=NULL && *flags=='f')
-					return 1;
+					return -1;
 			}
 		} else {
 			hfl = hf;
@@ -2855,6 +2999,7 @@ static int subst_hf_f(struct sip_msg *msg, char *str_hf, char *subst, char *flag
 			begin=body.s;
 
 			off=begin-msg->buf;
+			if (lst) replace_lst_free(lst);
 			lst=subst_run(se, begin, msg, &nmatches);
 			body.s[body.len] = c;
 			if(lst==0 && flags!=NULL && *flags=='f')
@@ -2891,7 +3036,7 @@ static int subst_hf_f(struct sip_msg *msg, char *str_hf, char *subst, char *flag
 		}
 		/* if flags set for first header, then all done */
 		if(flags!=NULL && *flags=='f')
-			return ret;
+			goto done;
 	}
 	if(hfl!=NULL)
 	{
@@ -2903,6 +3048,7 @@ static int subst_hf_f(struct sip_msg *msg, char *str_hf, char *subst, char *flag
 		begin=body.s;
 
 		off=begin-msg->buf;
+		if (lst) replace_lst_free(lst);
 		lst=subst_run(se, begin, msg, &nmatches);
 		body.s[body.len] = c;
 		if(lst==0)
@@ -2935,10 +3081,11 @@ static int subst_hf_f(struct sip_msg *msg, char *str_hf, char *subst, char *flag
 		}
 	}
 error:
-	LM_DBG("lst was %p\n", lst);
-	if (lst) replace_lst_free(lst);
 	if (nmatches<0)
 		LM_ERR("%s subst_run failed\n", exports.name);
+	LM_DBG("lst was %p\n", lst);
+done:
+	if (lst) replace_lst_free(lst);
 	return ret;
 }
 
