@@ -1,6 +1,4 @@
 /**
- * $Id$
- *
  * Copyright (C) 2010 Daniel-Constantin Mierla (asipto.com)
  *
  * This file is part of kamailio, a free SIP server.
@@ -44,15 +42,18 @@
 #include "auth_hdr.h"
 #include "uac_reg.h"
 
-#define UAC_REG_DISABLED	(1<<0)
-#define UAC_REG_ONGOING		(1<<1)
-#define UAC_REG_ONLINE		(1<<2)
-#define UAC_REG_AUTHSENT	(1<<3)
+#define UAC_REG_DISABLED	(1<<0) /* registration disabled */
+#define UAC_REG_ONGOING		(1<<1) /* registration on progress */
+#define UAC_REG_ONLINE		(1<<2) /* registered */
+#define UAC_REG_AUTHSENT	(1<<3) /* registration with auth in progress */
+#define UAC_REG_INIT		(1<<4) /* registration initialized */
 
 #define MAX_UACH_SIZE 2048
 #define UAC_REG_GC_INTERVAL	150
 #define UAC_REG_MAX_PASSWD_SIZE	63
 
+#define UAC_REG_MAX_URI_SIZE 127
+
 typedef struct _reg_uac
 {
 	unsigned int h_uuid;
@@ -69,6 +70,8 @@ typedef struct _reg_uac
 	unsigned int flags;
 	unsigned int expires;
 	time_t timer_expires;
+	unsigned int reg_delay;
+	time_t reg_init;
 } reg_uac_t;
 
 typedef struct _reg_item
@@ -117,23 +120,28 @@ str auth_username_column = str_init("auth_username");
 str auth_password_column = str_init("auth_password");
 str auth_proxy_column = str_init("auth_proxy");
 str expires_column = str_init("expires");
+str flags_column = str_init("flags");
+str reg_delay_column = str_init("reg_delay");
+
 
 #if 0
 INSERT INTO version (table_name, table_version) values ('uacreg','1');
 CREATE TABLE uacreg (
-    id INT(10) UNSIGNED AUTO_INCREMENT PRIMARY KEY NOT NULL,
-    l_uuid VARCHAR(64) NOT NULL,
-    l_username VARCHAR(64) NOT NULL,
-    l_domain VARCHAR(128) DEFAULT '' NOT NULL,
-    r_username VARCHAR(64) NOT NULL,
-    r_domain VARCHAR(128) NOT NULL,
-    realm VARCHAR(64) NOT NULL,
-    auth_username VARCHAR(64) NOT NULL,
-    auth_password VARCHAR(64) NOT NULL,
-    auth_proxy VARCHAR(128) DEFAULT '' NOT NULL,
-    expires INT(10) UNSIGNED DEFAULT 0 NOT NULL,
-    CONSTRAINT l_uuid_idx UNIQUE (l_uuid)
-) ENGINE=MyISAM;
+		id INT(10) UNSIGNED AUTO_INCREMENT PRIMARY KEY NOT NULL,
+		l_uuid VARCHAR(64) NOT NULL,
+		l_username VARCHAR(64) NOT NULL,
+		l_domain VARCHAR(128) DEFAULT '' NOT NULL,
+		r_username VARCHAR(64) NOT NULL,
+		r_domain VARCHAR(128) NOT NULL,
+		realm VARCHAR(64) NOT NULL,
+		auth_username VARCHAR(64) NOT NULL,
+		auth_password VARCHAR(64) NOT NULL,
+		auth_proxy VARCHAR(128) DEFAULT '' NOT NULL,
+		expires INT(10) UNSIGNED DEFAULT 0 NOT NULL,
+		flags INT(10) UNSIGNED DEFAULT 0 NOT NULL,
+		reg_delay INT(10) UNSIGNED DEFAULT 0 NOT NULL,
+		CONSTRAINT l_uuid_idx UNIQUE (l_uuid)
+		) ENGINE=MyISAM;
 #endif
 
 
@@ -142,9 +150,22 @@ extern pv_spec_t auth_username_spec;
 extern pv_spec_t auth_realm_spec;
 extern pv_spec_t auth_password_spec;
 
+counter_handle_t regtotal;         /* Total number of registrations in memory */
+counter_handle_t regactive;        /* Active registrations - 200 OK */
+counter_handle_t regdisabled;      /* Disabled registrations */
+
+/* Init counters */
+static void uac_reg_counter_init()
+{
+	LM_DBG("*** Initializing UAC reg counters\n");
+	counter_register(&regtotal, "uac", "regtotal", 0, 0, 0, "Total number of registration accounts in memory", 0);
+	counter_register(&regactive, "uac", "regactive", 0, 0, 0, "Number of successfully registred accounts (200 OK)", 0);
+	counter_register(&regdisabled, "uac", "regdisabled", 0, 0, 0, "Counter of failed registrations (not 200 OK)", 0);
+}
+
 
 /**
- *
+ * Init the in-memory registration database in hash table
  */
 int uac_reg_init_ht(unsigned int sz)
 {
@@ -174,7 +195,7 @@ int uac_reg_init_ht(unsigned int sz)
 	_reg_htable_gc->htsize = sz;
 
 	_reg_htable_gc->entries =
-			(reg_entry_t*)shm_malloc(_reg_htable_gc->htsize*sizeof(reg_entry_t));
+		(reg_entry_t*)shm_malloc(_reg_htable_gc->htsize*sizeof(reg_entry_t));
 	if(_reg_htable_gc->entries==NULL)
 	{
 		LM_ERR("no more shm.\n");
@@ -200,7 +221,7 @@ int uac_reg_init_ht(unsigned int sz)
 	_reg_htable->htsize = sz;
 
 	_reg_htable->entries =
-			(reg_entry_t*)shm_malloc(_reg_htable->htsize*sizeof(reg_entry_t));
+		(reg_entry_t*)shm_malloc(_reg_htable->htsize*sizeof(reg_entry_t));
 	if(_reg_htable->entries==NULL)
 	{
 		LM_ERR("no more shm.\n");
@@ -216,7 +237,7 @@ int uac_reg_init_ht(unsigned int sz)
 	{
 		if(lock_init(&_reg_htable->entries[i].lock)==0)
 		{
-			LM_ERR("cannot initalize lock[%d] n", i);
+			LM_ERR("cannot initialize lock[%d] n", i);
 			i--;
 			while(i>=0)
 			{
@@ -233,6 +254,9 @@ int uac_reg_init_ht(unsigned int sz)
 		}
 	}
 
+	/* Initialize uac reg counters */
+	uac_reg_counter_init();
+
 	return 0;
 }
 
@@ -345,6 +369,10 @@ int uac_reg_reset_ht_gc(void)
 		_reg_htable_gc->entries[i].byuser = NULL;
 		_reg_htable_gc->entries[i].usize = 0;
 	}
+	/* Reset all counters */
+	counter_reset(regtotal);
+	counter_reset(regactive);
+	counter_reset(regdisabled);
 	return 0;
 }
 
@@ -366,7 +394,7 @@ int uac_reg_ht_shift(void)
 	lock_get(_reg_htable_gc_lock);
 	if(_reg_htable_gc->stime > tn-UAC_REG_GC_INTERVAL) {
 		lock_release(_reg_htable_gc_lock);
-		LM_ERR("shifting the memory table is not possible in less than %d\n", UAC_REG_GC_INTERVAL);
+		LM_ERR("shifting the memory table is not possible in less than %d secs\n", UAC_REG_GC_INTERVAL);
 		return -1;
 	}
 	uac_reg_reset_ht_gc();
@@ -452,13 +480,13 @@ int reg_ht_add_byuser(reg_uac_t *reg)
 	return 0;
 }
 
-#define reg_copy_shm(dst, src) do { \
+#define reg_copy_shm(dst, src, bsize) do { \
 		if((src)->s!=NULL) { \
 			(dst)->s = p; \
 			strncpy((dst)->s, (src)->s, (src)->len); \
 			(dst)->len = (src)->len; \
 			(dst)->s[(dst)->len] = '\0'; \
-			p = p + (dst)->len + 1; \
+			p = p + ((bsize)?(bsize):(dst)->len) + 1; \
 		} \
 	} while(0);
 
@@ -482,16 +510,22 @@ int reg_ht_add(reg_uac_t *reg)
 				reg, _reg_htable, reg->auth_password.len);
 		return -1;
 	}
+	if(reg->auth_proxy.len>UAC_REG_MAX_URI_SIZE)
+	{
+		LM_ERR("bad parameters: %p/%p -- proxy uri too long %d\n",
+				reg, _reg_htable, reg->auth_proxy.len);
+		return -1;
+	}
 
 	len = reg->l_uuid.len + 1
-			+ reg->l_username.len + 1
-			+ reg->l_domain.len + 1
-			+ reg->r_username.len + 1
-			+ reg->r_domain.len + 1
-			+ reg->realm.len + 1
-			+ reg->auth_proxy.len + 1
-			+ reg->auth_username.len + 1
-			+ UAC_REG_MAX_PASSWD_SIZE /*reg->auth_password.len*/ + 1;
+		+ reg->l_username.len + 1
+		+ reg->l_domain.len + 1
+		+ reg->r_username.len + 1
+		+ reg->r_domain.len + 1
+		+ reg->realm.len + 1
+		+ UAC_REG_MAX_URI_SIZE /*reg->auth_proxy.len*/ + 1
+		+ reg->auth_username.len + 1
+		+ UAC_REG_MAX_PASSWD_SIZE /*reg->auth_password.len*/ + 1;
 	nr = (reg_uac_t*)shm_malloc(sizeof(reg_uac_t) + len);
 	if(nr==NULL)
 	{
@@ -500,24 +534,28 @@ int reg_ht_add(reg_uac_t *reg)
 	}
 	memset(nr, 0, sizeof(reg_uac_t) + len);
 	nr->expires = reg->expires;
+	nr->flags   = reg->flags;
+	nr->reg_delay = reg->reg_delay;
+	nr->reg_init  = time(NULL);
 	nr->h_uuid = reg_compute_hash(&reg->l_uuid);
 	nr->h_user = reg_compute_hash(&reg->l_username);
-	
+
 	p = (char*)nr + sizeof(reg_uac_t);
 
-	reg_copy_shm(&nr->l_uuid, &reg->l_uuid);
-	reg_copy_shm(&nr->l_username, &reg->l_username);
-	reg_copy_shm(&nr->l_domain, &reg->l_domain);
-	reg_copy_shm(&nr->r_username, &reg->r_username);
-	reg_copy_shm(&nr->r_domain, &reg->r_domain);
-	reg_copy_shm(&nr->realm, &reg->realm);
-	reg_copy_shm(&nr->auth_proxy, &reg->auth_proxy);
-	reg_copy_shm(&nr->auth_username, &reg->auth_username);
+	reg_copy_shm(&nr->l_uuid, &reg->l_uuid, 0);
+	reg_copy_shm(&nr->l_username, &reg->l_username, 0);
+	reg_copy_shm(&nr->l_domain, &reg->l_domain, 0);
+	reg_copy_shm(&nr->r_username, &reg->r_username, 0);
+	reg_copy_shm(&nr->r_domain, &reg->r_domain, 0);
+	reg_copy_shm(&nr->realm, &reg->realm, 0);
+	reg_copy_shm(&nr->auth_proxy, &reg->auth_proxy, UAC_REG_MAX_URI_SIZE);
+	reg_copy_shm(&nr->auth_username, &reg->auth_username, 0);
 	/* password at the end, to be able to update it easily */
-	reg_copy_shm(&nr->auth_password, &reg->auth_password);
+	reg_copy_shm(&nr->auth_password, &reg->auth_password, 0);
 
 	reg_ht_add_byuser(nr);
 	reg_ht_add_byuuid(nr);
+	counter_inc(regtotal);
 
 	return 0;
 }
@@ -526,7 +564,7 @@ int reg_ht_add(reg_uac_t *reg)
 /**
  *
  */
-int reg_ht_update_password(reg_uac_t *reg)
+int reg_ht_update_attrs(reg_uac_t *reg)
 {
 	unsigned int slot;
 	reg_item_t *ri = NULL;
@@ -542,6 +580,11 @@ int reg_ht_update_password(reg_uac_t *reg)
 		LM_ERR("password is too big: %d\n", reg->auth_password.len);
 		return -1;
 	}
+	if(reg->auth_proxy.len>UAC_REG_MAX_URI_SIZE)
+	{
+		LM_ERR("proxy uri is too big: %d\n", reg->auth_proxy.len);
+		return -1;
+	}
 
 	slot = reg_get_entry(reg->h_user, _reg_htable->htsize);
 	lock_get(&_reg_htable->entries[slot].lock);
@@ -554,6 +597,10 @@ int reg_ht_update_password(reg_uac_t *reg)
 			strncpy(ri->r->auth_password.s, reg->auth_password.s, reg->auth_password.len);
 			ri->r->auth_password.len = reg->auth_password.len;
 			ri->r->auth_password.s[reg->auth_password.len] = '\0';
+			strncpy(ri->r->auth_proxy.s, reg->auth_proxy.s, reg->auth_proxy.len);
+			ri->r->auth_proxy.len = reg->auth_proxy.len;
+			ri->r->auth_proxy.s[reg->auth_proxy.len] = '\0';
+			if(reg->flags & UAC_REG_DISABLED) ri->r->flags |= UAC_REG_DISABLED;
 			lock_release(&_reg_htable->entries[slot].lock);
 			return 0;
 		}
@@ -619,7 +666,7 @@ reg_uac_t *reg_ht_get_byuser(str *user, str *domain)
 			if(domain!=NULL && domain->s!=NULL)
 			{
 				if((it->r->l_domain.len==domain->len)
-				&& (strncmp(it->r->l_domain.s, domain->s, domain->len)==0))
+						&& (strncmp(it->r->l_domain.s, domain->s, domain->len)==0))
 				{
 					return it->r;
 				}
@@ -700,7 +747,7 @@ void uac_reg_tm_callback( struct cell *t, int type, struct tmcb_params *ps)
 	}
 	uuid = *((char**)ps->param);
 	LM_DBG("completed with status %d [uuid: %s]\n",
-		ps->code, uuid);
+			ps->code, uuid);
 	suuid.s = uuid;
 	suuid.len = strlen(suuid.s);
 	ri = reg_ht_get_byuuid(&suuid);
@@ -783,7 +830,7 @@ void uac_reg_tm_callback( struct cell *t, int type, struct tmcb_params *ps)
 		}
 
 		LM_DBG("auth header body [%.*s]\n",
-			hdr->body.len, hdr->body.s);
+				hdr->body.len, hdr->body.s);
 
 		if (parse_authenticate_body(&hdr->body, &auth)<0)
 		{
@@ -796,14 +843,14 @@ void uac_reg_tm_callback( struct cell *t, int type, struct tmcb_params *ps)
 					|| strncmp(auth.realm.s, ri->realm.s, ri->realm.len)!=0)
 			{
 				LM_ERR("realms do not match. requested realm: [%.*s]\n",
-					auth.realm.len, auth.realm.s);
+						auth.realm.len, auth.realm.s);
 				goto error;
 			}
 		}
 		cred.realm = auth.realm;
 		cred.user = ri->auth_username; 
 		cred.passwd = ri->auth_password;
- 		cred.next = NULL;
+		cred.next = NULL;
 
 		snprintf(b_ruri, MAX_URI_SIZE, "sip:%.*s",
 				ri->r_domain.len, ri->r_domain.s);
@@ -811,13 +858,13 @@ void uac_reg_tm_callback( struct cell *t, int type, struct tmcb_params *ps)
 
 		do_uac_auth(&method, &s_ruri, &cred, &auth, response);
 		new_auth_hdr=build_authorization_hdr(ps->code, &s_ruri, &cred,
-						&auth, response);
+				&auth, response);
 		if (new_auth_hdr==0)
 		{
 			LM_ERR("failed to build authorization hdr\n");
 			goto error;
 		}
-		
+
 #ifdef UAC_OLD_AUTH
 		snprintf(b_turi, MAX_URI_SIZE, "sip:%.*s@%.*s",
 				ri->r_username.len, ri->r_username.s,
@@ -858,13 +905,13 @@ void uac_reg_tm_callback( struct cell *t, int type, struct tmcb_params *ps)
 				&s_turi, /* To */
 				&s_turi, /* From */
 				(ri->auth_proxy.len)?&ri->auth_proxy:NULL /* outbound uri */
-			);
+				);
 #endif
 		ret = uac_tmb.t_request_within(&uac_r);
 
 		if(ret<0) {
 			LM_ERR("failed to send request with authentication for [%.*s]",
-			       ri->l_uuid.len, ri->l_uuid.s);
+					ri->l_uuid.len, ri->l_uuid.s);
 			goto error;
 		}
 
@@ -873,19 +920,22 @@ void uac_reg_tm_callback( struct cell *t, int type, struct tmcb_params *ps)
 	} else
 	{
 		LM_ERR("got sip response %d while registering [%.*s]\n",
-		       ps->code, ri->l_uuid.len, ri->l_uuid.s);
+				ps->code, ri->l_uuid.len, ri->l_uuid.s);
 		goto error;
 	}
 
 error:
-	if(reg_retry_interval)
+	if(reg_retry_interval) {
 		ri->timer_expires = time(NULL) + reg_retry_interval;
-	else
+	} else {
 		ri->flags |= UAC_REG_DISABLED;
+		counter_inc(regdisabled);
+	}
 done:
 	if(ri)
 		ri->flags &= ~(UAC_REG_ONGOING|UAC_REG_AUTHSENT);
 	shm_free(uuid);
+	counter_inc(regactive);
 }
 
 int uac_reg_update(reg_uac_t *reg, time_t tn)
@@ -905,21 +955,37 @@ int uac_reg_update(reg_uac_t *reg, time_t tn)
 		return -1;
 	if(reg->expires==0)
 		return 1;
-	if(reg->flags&UAC_REG_ONGOING)
-		return 2;
+	if(reg->flags&UAC_REG_ONGOING) {
+		if (reg->timer_expires > tn - reg_retry_interval)
+			return 2;
+		LM_DBG("record marked as ongoing registration (%d) - resetting\n",
+				(int)reg->flags);
+		reg->flags &= ~(UAC_REG_ONLINE|UAC_REG_AUTHSENT);
+	}
 	if(reg->flags&UAC_REG_DISABLED)
 		return 4;
+
+	if(!(reg->flags & UAC_REG_INIT)) {
+		if(reg->reg_delay>0) {
+			if(tn < reg->reg_init+reg->reg_delay) {
+				return 2;
+			}
+		}
+	}
+	reg->flags |= UAC_REG_INIT;
+
 	if(reg->timer_expires > tn + reg_timer_interval + 3)
 		return 3;
-	reg->timer_expires = tn;
-	reg->flags |= UAC_REG_ONGOING;
-	reg->flags &= ~UAC_REG_ONLINE;
 	uuid = (char*)shm_malloc(reg->l_uuid.len+1);
 	if(uuid==NULL)
 	{
 		LM_ERR("no more shm\n");
 		return -1;
 	}
+	reg->timer_expires = tn;
+	reg->flags |= UAC_REG_ONGOING;
+	reg->flags &= ~UAC_REG_ONLINE;
+	counter_add(regactive, -1);		/* Take it out of the active pool while re-registering */
 	memcpy(uuid, reg->l_uuid.s, reg->l_uuid.len);
 	uuid[reg->l_uuid.len] = '\0';
 
@@ -953,7 +1019,7 @@ int uac_reg_update(reg_uac_t *reg, time_t tn)
 			&s_turi, /* To */
 			&s_turi, /* From */
 			(reg->auth_proxy.len)?&reg->auth_proxy:NULL /* outbound uri */
-		);
+			);
 
 	if(ret<0)
 	{
@@ -961,8 +1027,10 @@ int uac_reg_update(reg_uac_t *reg, time_t tn)
 		shm_free(uuid);
 		if (reg_retry_interval)
 			reg->timer_expires = (tn ? tn : time(NULL)) + reg_retry_interval;
-		else
+		else {
 			reg->flags |= UAC_REG_DISABLED;
+			counter_inc(regdisabled);
+		}
 		reg->flags &= ~UAC_REG_ONGOING;
 		return -1;
 	}
@@ -1004,17 +1072,17 @@ void uac_reg_timer(unsigned int ticks)
 }
 
 #define reg_db_set_attr(attr, pos) do { \
-		if(!VAL_NULL(&RES_ROWS(db_res)[i].values[pos])) { \
-			reg.attr.s = \
-				(char*)(RES_ROWS(db_res)[i].values[pos].val.string_val); \
-			reg.attr.len = strlen(reg.attr.s); \
-			if(reg.attr.len == 0) { \
-				LM_ERR("empty value not allowed for column[%d]='%.*s' - ignoring record\n", \
-						pos, db_cols[pos]->len, db_cols[pos]->s); \
-				continue; \
-			} \
+	if(!VAL_NULL(&RES_ROWS(db_res)[i].values[pos])) { \
+		reg.attr.s = \
+		(char*)(RES_ROWS(db_res)[i].values[pos].val.string_val); \
+		reg.attr.len = strlen(reg.attr.s); \
+		if(reg.attr.len == 0) { \
+			LM_ERR("empty value not allowed for column[%d]='%.*s' - ignoring record\n", \
+					pos, db_cols[pos]->len, db_cols[pos]->s); \
+			continue; \
 		} \
-	} while(0);
+	} \
+} while(0);
 
 /**
  *
@@ -1024,7 +1092,7 @@ int uac_reg_load_db(void)
 	db1_con_t *reg_db_con = NULL;
 	db_func_t reg_dbf;
 	reg_uac_t reg;
-	db_key_t db_cols[10] = {
+	db_key_t db_cols[12] = {
 		&l_uuid_column,
 		&l_username_column,
 		&l_domain_column,
@@ -1034,7 +1102,9 @@ int uac_reg_load_db(void)
 		&auth_username_column,
 		&auth_password_column,
 		&auth_proxy_column,
-		&expires_column
+		&expires_column,
+		&flags_column,
+		&reg_delay_column
 	};
 	db1_res_t* db_res = NULL;
 	int i, ret;
@@ -1055,7 +1125,7 @@ int uac_reg_load_db(void)
 	if (!DB_CAPABILITY(reg_dbf, DB_CAP_ALL))
 	{
 		LM_ERR("database module does not "
-		    "implement all functions needed by the module\n");
+				"implement all functions needed by the module\n");
 		return -1;
 	}
 
@@ -1073,7 +1143,7 @@ int uac_reg_load_db(void)
 	}
 
 	if (DB_CAPABILITY(reg_dbf, DB_CAP_FETCH)) {
-		if(reg_dbf.query(reg_db_con, 0, 0, 0, db_cols, 0, 10, 0, 0) < 0)
+		if(reg_dbf.query(reg_db_con, 0, 0, 0, db_cols, 0, 12, 0, 0) < 0)
 		{
 			LM_ERR("Error while querying db\n");
 			return -1;
@@ -1092,8 +1162,8 @@ int uac_reg_load_db(void)
 		}
 	} else {
 		if((ret=reg_dbf.query(reg_db_con, NULL, NULL, NULL, db_cols,
-				0, 10, 0, &db_res))!=0
-			|| RES_ROW_N(db_res)<=0 )
+						0, 10, 0, &db_res))!=0
+				|| RES_ROW_N(db_res)<=0 )
 		{
 			reg_dbf.free_result(reg_db_con, db_res);
 			if( ret==0)
@@ -1125,13 +1195,17 @@ int uac_reg_load_db(void)
 			reg_db_set_attr(auth_proxy, 8);
 			reg.expires
 				= (unsigned int)RES_ROWS(db_res)[i].values[9].val.int_val;
-			
+			reg.flags
+				= (unsigned int)RES_ROWS(db_res)[i].values[10].val.int_val;
+			reg.reg_delay
+				= (unsigned int)RES_ROWS(db_res)[i].values[11].val.int_val;
+
 			if(reg_ht_add(&reg)<0)
 			{
 				LM_ERR("Error adding reg to htable\n");
 				goto error;
 			}
-	 	}
+		}
 		if (DB_CAPABILITY(reg_dbf, DB_CAP_FETCH)) {
 			if(reg_dbf.fetch_result(reg_db_con, &db_res, reg_fetch_rows)<0) {
 				LM_ERR("Error while fetching!\n");
@@ -1165,7 +1239,7 @@ int uac_reg_db_refresh(str *pl_uuid)
 	db1_con_t *reg_db_con = NULL;
 	db_func_t reg_dbf;
 	reg_uac_t reg;
-	db_key_t db_cols[10] = {
+	db_key_t db_cols[12] = {
 		&l_uuid_column,
 		&l_username_column,
 		&l_domain_column,
@@ -1175,7 +1249,9 @@ int uac_reg_db_refresh(str *pl_uuid)
 		&auth_username_column,
 		&auth_password_column,
 		&auth_proxy_column,
-		&expires_column
+		&expires_column,
+		&flags_column,
+		&reg_delay_column
 	};
 	db_key_t db_keys[1] = {&l_uuid_column};
 	db_val_t db_vals[1];
@@ -1199,7 +1275,7 @@ int uac_reg_db_refresh(str *pl_uuid)
 	if (!DB_CAPABILITY(reg_dbf, DB_CAP_ALL))
 	{
 		LM_ERR("database module does not "
-		    "implement all functions needed by the module\n");
+				"implement all functions needed by the module\n");
 		return -1;
 	}
 
@@ -1222,8 +1298,8 @@ int uac_reg_db_refresh(str *pl_uuid)
 	db_vals[0].val.str_val.len = pl_uuid->len;
 
 	if((ret=reg_dbf.query(reg_db_con, db_keys, NULL, db_vals, db_cols,
-			1 /*nr keys*/, 10 /*nr cols*/, 0, &db_res))!=0
-		|| RES_ROW_N(db_res)<=0 )
+					1 /*nr keys*/, 12 /*nr cols*/, 0, &db_res))!=0
+			|| RES_ROW_N(db_res)<=0 )
 	{
 		reg_dbf.free_result(reg_db_con, db_res);
 		if( ret==0)
@@ -1253,11 +1329,13 @@ int uac_reg_db_refresh(str *pl_uuid)
 	reg.expires = (unsigned int)RES_ROWS(db_res)[i].values[9].val.int_val;
 	reg.h_uuid = reg_compute_hash(&reg.l_uuid);
 	reg.h_user = reg_compute_hash(&reg.l_username);
+	reg.flags = (unsigned int)RES_ROWS(db_res)[i].values[10].val.int_val;
+	reg.reg_delay = (unsigned int)RES_ROWS(db_res)[i].values[11].val.int_val;
 
 	lock_get(_reg_htable_gc_lock);
 	if(reg_ht_get_byuuid(pl_uuid)!=NULL)
 	{
-		if(reg_ht_update_password(&reg)<0)
+		if(reg_ht_update_attrs(&reg)<0)
 		{
 			lock_release(_reg_htable_gc_lock);
 			LM_ERR("Error updating reg to htable\n");
@@ -1311,8 +1389,8 @@ int  uac_reg_lookup(struct sip_msg *msg, str *src, pv_spec_t *dst, int mode)
 			return -1;
 		}
 		snprintf(b_ruri, MAX_URI_SIZE, "sip:%.*s@%.*s",
-			reg->l_username.len, reg->l_username.s,
-			reg->l_domain.len, reg->l_domain.s);
+				reg->l_username.len, reg->l_username.s,
+				reg->l_domain.len, reg->l_domain.s);
 		s_ruri.s = b_ruri; s_ruri.len = strlen(s_ruri.s);
 	} else {
 		if(parse_uri(src->s, src->len, &puri)!=0)
@@ -1327,7 +1405,7 @@ int  uac_reg_lookup(struct sip_msg *msg, str *src, pv_spec_t *dst, int mode)
 			return -1;
 		}
 		snprintf(b_ruri, MAX_URI_SIZE, "%.*s",
-			reg->l_uuid.len, reg->l_uuid.s);
+				reg->l_uuid.len, reg->l_uuid.s);
 		s_ruri.s = b_ruri; s_ruri.len = strlen(s_ruri.s);
 	}
 	memset(&val, 0, sizeof(pv_value_t));
@@ -1382,8 +1460,8 @@ int uac_reg_request_to(struct sip_msg *msg, str *src, unsigned int mode)
 
 	// Set uri ($ru)
 	snprintf(ruri, MAX_URI_SIZE, "sip:%.*s@%.*s",
-		reg->r_username.len, reg->r_username.s,
-		reg->r_domain.len, reg->r_domain.s);
+			reg->r_username.len, reg->r_username.s,
+			reg->r_domain.len, reg->r_domain.s);
 	memset(&act, 0, sizeof(act));
 	act.type = SET_URI_T;
 	act.val[0].type = STRING_ST;
@@ -1461,22 +1539,24 @@ static void rpc_uac_reg_dump(rpc_t* rpc, void* ctx)
 				rpc->fault(ctx, 500, "Internal error creating rpc");
 				return;
 			}
-			if(rpc->struct_add(th, "SSSSSSSSSdddd",
-					"l_uuid",        &reg->r->l_uuid,
-					"l_username",    &reg->r->l_username,
-					"l_domain",      &reg->r->l_domain,
-					"r_username",    &reg->r->r_username,
-					"r_domain",      &reg->r->r_domain,
-					"realm",         &reg->r->realm,
-					"auth_username", &reg->r->auth_username,
-					"auth_password", &reg->r->auth_password,
-					"auth_proxy",    (reg->r->auth_proxy.len)?
-										&reg->r->auth_proxy:&none,
-					"expires",       (int)reg->r->expires,
-					"flags",         (int)reg->r->flags,
-					"diff_expires",  (int)(reg->r->timer_expires - tn),
-					"timer_expires", (int)reg->r->timer_expires
-				)<0)
+			if(rpc->struct_add(th, "SSSSSSSSSdddddd",
+						"l_uuid",        &reg->r->l_uuid,
+						"l_username",    &reg->r->l_username,
+						"l_domain",      &reg->r->l_domain,
+						"r_username",    &reg->r->r_username,
+						"r_domain",      &reg->r->r_domain,
+						"realm",         &reg->r->realm,
+						"auth_username", &reg->r->auth_username,
+						"auth_password", &reg->r->auth_password,
+						"auth_proxy",    (reg->r->auth_proxy.len)?
+						&reg->r->auth_proxy:&none,
+						"expires",       (int)reg->r->expires,
+						"flags",         (int)reg->r->flags,
+						"diff_expires",  (int)(reg->r->timer_expires - tn),
+						"timer_expires", (int)reg->r->timer_expires,
+						"reg_init",      (int)reg->r->reg_init,
+						"reg_delay",     (int)reg->r->reg_delay
+						)<0)
 			{
 				lock_release(&_reg_htable->entries[i].lock);
 				rpc->fault(ctx, 500, "Internal error adding item");
@@ -1554,21 +1634,21 @@ static void rpc_uac_reg_info(rpc_t* rpc, void* ctx)
 					return;
 				}
 				if(rpc->struct_add(th, "SSSSSSSSSdddd",
-						"l_uuid",        &reg->r->l_uuid,
-						"l_username",    &reg->r->l_username,
-						"l_domain",      &reg->r->l_domain,
-						"r_username",    &reg->r->r_username,
-						"r_domain",      &reg->r->r_domain,
-						"realm",         &reg->r->realm,
-						"auth_username", &reg->r->auth_username,
-						"auth_password", &reg->r->auth_password,
-						"auth_proxy",    (reg->r->auth_proxy.len)?
-											&reg->r->auth_proxy:&none,
-						"expires",       (int)reg->r->expires,
-						"flags",         (int)reg->r->flags,
-						"diff_expires",  (int)(reg->r->timer_expires - tn),
-						"timer_expires", (int)reg->r->timer_expires
-					)<0)
+							"l_uuid",        &reg->r->l_uuid,
+							"l_username",    &reg->r->l_username,
+							"l_domain",      &reg->r->l_domain,
+							"r_username",    &reg->r->r_username,
+							"r_domain",      &reg->r->r_domain,
+							"realm",         &reg->r->realm,
+							"auth_username", &reg->r->auth_username,
+							"auth_password", &reg->r->auth_password,
+							"auth_proxy",    (reg->r->auth_proxy.len)?
+							&reg->r->auth_proxy:&none,
+							"expires",       (int)reg->r->expires,
+							"flags",         (int)reg->r->flags,
+							"diff_expires",  (int)(reg->r->timer_expires - tn),
+							"timer_expires", (int)reg->r->timer_expires
+							)<0)
 				{
 					lock_release(&_reg_htable->entries[i].lock);
 					rpc->fault(ctx, 500, "Internal error adding item");
@@ -1657,6 +1737,7 @@ static const char* rpc_uac_reg_enable_doc[2] = {
 static void rpc_uac_reg_enable(rpc_t* rpc, void* ctx)
 {
 	rpc_uac_reg_update_flag(rpc, ctx, 0, UAC_REG_DISABLED);
+	counter_add(regdisabled, -1);
 }
 
 static const char* rpc_uac_reg_disable_doc[2] = {
@@ -1667,6 +1748,7 @@ static const char* rpc_uac_reg_disable_doc[2] = {
 static void rpc_uac_reg_disable(rpc_t* rpc, void* ctx)
 {
 	rpc_uac_reg_update_flag(rpc, ctx, 1, UAC_REG_DISABLED);
+	counter_inc(regdisabled);
 }
 
 static const char* rpc_uac_reg_reload_doc[2] = {
@@ -1714,7 +1796,7 @@ static void rpc_uac_reg_refresh(rpc_t* rpc, void* ctx)
 }
 
 rpc_export_t uac_reg_rpc[] = {
-	{"uac.reg_dump", rpc_uac_reg_dump, rpc_uac_reg_dump_doc, 0},
+	{"uac.reg_dump", rpc_uac_reg_dump, rpc_uac_reg_dump_doc, RET_ARRAY},
 	{"uac.reg_info", rpc_uac_reg_info, rpc_uac_reg_info_doc, 0},
 	{"uac.reg_enable",  rpc_uac_reg_enable,  rpc_uac_reg_enable_doc,  0},
 	{"uac.reg_disable", rpc_uac_reg_disable, rpc_uac_reg_disable_doc, 0},
