@@ -1,22 +1,14 @@
 /*
- * $Id$
- *
- *
  * Copyright (C) 2001-2003 FhG Fokus
  *
- * This file is part of ser, a free SIP server.
+ * This file is part of Kamailio, a free SIP server.
  *
- * ser is free software; you can redistribute it and/or modify
+ * Kamailio is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version
  *
- * For a license to use the ser software under conditions
- * other than those described here, or to purchase support for this
- * software, please contact iptel.org by e-mail at the following addresses:
- *    info@iptel.org
- *
- * ser is distributed in the hope that it will be useful,
+ * Kamailio is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -26,48 +18,9 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  *
- * History:
- * --------
- * 2003-01-20  bug_fix: use of return value of snprintf aligned to C99 (jiri)
- * 2003-01-23  added rport patches, contributed by
- *              Maxim Sobolev <sobomax@FreeBSD.org> and heavily modified by me
- *              (andrei)
- * 2003-01-24  added i param to via of outgoing requests (used by tcp),
- *              modified via_builder params (andrei)
- * 2003-01-27  more rport fixes (make use of new via_param->start)  (andrei)
- * 2003-01-27  next baby-step to removing ZT - PRESERVE_ZT (jiri)
- * 2003-01-29  scratchpad removed (jiri)
- * 2003-02-28  scratchpad compatibility abandoned (jiri)
- * 2003-03-01  VOICE_MAIL defs removed (jiri)
- * 2003-03-06  totags in outgoing replies bookmarked to enable
- *             ACK/200 tag matching (andrei)
- * 2003-03-18  killed the build_warning snprintf (andrei)
- * 2003-03-31  added subst lump support (andrei)
- * 2003-04-01  added opt (conditional) lump support (andrei)
- * 2003-04-02  added more subst lumps: SUBST_{SND,RCV}_ALL
- *              => ip:port;transport=proto (andrei)
- * 2003-04-12  added FL_FORCE_RPORT support (andrei)
- * 2003-04-13  updated warning builder -- fixed (andrei)
- * 2003-07-10  check_via_address knows now how to compare with ipv6 address
- *              references (e.g [::1]) (andrei)
- *             build_req_fomr_sip_req no longer adds 1 for ipv6 via parameter
- *              position calculations ([] are part of host.s now) (andrei)
- * 2003-10-02  via+lump dst address/port can be set to preset values (andrei)
- * 2003-10-08  receive_test function-alized (jiri)
- * 2003-10-20  added body_lump list (sip_msg), adjust_clen (andrei & jan)
- * 2003-11-11  type of rpl_lumps replaced by flags (bogdan)
- * 2006-04-20  build_req_from_sip_req, via_builder and lump_* functions
- *              use now struct dest_info; lumps & via comp param support
- *              (rfc3486) (andrei)
- * 2007-08-31  id_builder() and via_builder() are grouped into one function:
- *             create_via_hf() -- tm module needs them as well (Miklos)
- * 2008-12-17  build_req_from_sip_req() will now fallback to tcp, tls or sctp
- *              if packet size > udp_mtu and fallback is enabled 
- *             build_req_from_sip_req() uses now global_req_flags along
- *               msg->msg_flags  (andrei)
  *
  */
-/* Via special params:
+/** Via special params:
  * requests:
  * - if the address in via is different from the src_ip or an existing
  *   received=something is found, received=src_ip is added (and any preexisting
@@ -119,7 +72,7 @@
 
 /*!
  * \file
- * \brief SIP-router core :: 
+ * \brief Kamailio core :: Message translations
  * \ingroup core
  * Module: \ref core
  */
@@ -202,8 +155,7 @@ static int check_via_address(struct ip_addr* ip, str *name,
 	/* maybe we are lucky and name it's an ip */
 	s=ip_addr2a(ip);
 	if (s){
-		DBG("check_via_address(%s, %.*s, %d)\n",
-			s, name->len, name->s, resolver);
+		LM_DBG("(%s, %.*s, %d)\n", s, name->len, name->s, resolver);
 
 		len=strlen(s);
 
@@ -228,7 +180,7 @@ static int check_via_address(struct ip_addr* ip, str *name,
 
 	if (port==0) port=SIP_PORT;
 	if (resolver&DO_DNS){
-		DBG("check_via_address: doing dns lookup\n");
+		LM_DBG("doing dns lookup\n");
 		/* try all names ips */
 		lproto = PROTO_NONE;
 		he=sip_resolvehost(name, &port, &lproto); /* don't use naptr */
@@ -240,7 +192,7 @@ static int check_via_address(struct ip_addr* ip, str *name,
 		}
 	}
 	if (resolver&DO_REV_DNS){
-		DBG("check_via_address: doing rev. dns lookup\n");
+		LM_DBG("doing rev. dns lookup\n");
 		/* try reverse dns */
 		he=rev_resolvehost(ip);
 		if (he && (strncmp(he->h_name, name->s, name->len)==0))
@@ -467,7 +419,7 @@ char* clen_builder(	struct sip_msg* msg, int *clen_len, int diff,
 	}
 	value=msg->len-(int)(body-msg->buf)+diff;
 	value_s=int2str(value, &value_len);
-	DBG("clen_builder: content-length: %d (%s)\n", value, value_s);
+	LM_DBG("content-length: %d (%s)\n", value, value_s);
 
 	if (body_only) {
 		len=value_len;
@@ -938,12 +890,13 @@ skip_after:
 /* another helper functions, adds/Removes the lump,
 	code moved form build_req_from_req  */
 
-static inline void process_lumps(	struct sip_msg* msg,
-					                                struct lump* lumps,
-									char* new_buf,
-									unsigned int* new_buf_offs,
-									unsigned int* orig_offs,
-									struct dest_info* send_info)
+void process_lumps( struct sip_msg* msg,
+                    struct lump* lumps,
+                    char* new_buf,
+                    unsigned int* new_buf_offs,
+                    unsigned int* orig_offs,
+                    struct dest_info* send_info,
+                    int flag)
 {
 	struct lump *t;
 	struct lump *r;
@@ -1397,18 +1350,28 @@ skip_after:
 			case LUMP_DEL:
 				/* copy till offset */
 				if (s_offset>t->u.offset){
-					DBG("Warning: (%d) overlapped lumps offsets,"
+					LM_DBG("WARNING: (%d) overlapped lumps offsets,"
 						" ignoring(%x, %x)\n", t->op, s_offset,t->u.offset);
 					/* this should've been fixed above (when computing len) */
 					/* just ignore it*/
 					break;
 				}
 				size=t->u.offset-s_offset;
-				if (size){
+                if (size > 0 && flag == FLAG_MSG_ALL){
 					memcpy(new_buf+offset, orig+s_offset,size);
 					offset+=size;
 					s_offset+=size;
-				}
+                } else if (flag == FLAG_MSG_LUMPS_ONLY) {
+                    /* do not copy the whole message, jump to the lumps offs */
+                    s_offset+=size;
+                }
+
+                /* the LUMP_DELs are printed with "- " before them */
+                if (t->op==LUMP_DEL && flag == FLAG_MSG_LUMPS_ONLY) {
+                    new_buf[offset++] = '-';
+                    new_buf[offset++] = ' ';
+                }
+
 				/* process before  */
 				for(r=t->before;r;r=r->before){
 					switch (r->op){
@@ -1432,11 +1395,22 @@ skip_after:
 					}
 				}
 skip_nop_before:
-				/* process main (del only) */
-				if (t->op==LUMP_DEL){
-					/* skip len bytes from orig msg */
-					s_offset+=t->len;
-				}
+                /* process main (del only) */
+                if (t->op==LUMP_DEL && flag == FLAG_MSG_ALL){
+                    /* skip len bytes from orig msg */
+                    s_offset+=t->len;
+                } else if (t->op==LUMP_DEL && flag == FLAG_MSG_LUMPS_ONLY) {
+                    /* copy lump value and indent as necessarely */
+                    memcpy(new_buf+offset, orig + t->u.offset, t->len);
+                    offset+=t->len;
+                    if (new_buf[offset-1] != '\n') {
+                        new_buf[offset] = '\n';
+                        offset+=1;
+                    }
+                    /* skip len bytes from orig msg */
+                    s_offset+=t->len;
+                 }
+
 				/* process after */
 				for(r=t->after;r;r=r->after){
 					switch (r->op){
@@ -1467,6 +1441,11 @@ skip_nop_after:
 	}
 	*new_buf_offs=offset;
 	*orig_offs=s_offset;
+
+    /* add '\0' to char* lump list to print it smoothly */
+    if (flag == FLAG_MSG_LUMPS_ONLY) {
+        new_buf[offset] = '\0';
+    }
 #undef RCVCOMP_PARAM_ADD 
 #undef SENDCOMP_PARAM_ADD
 }
@@ -1570,8 +1549,7 @@ static inline int adjust_clen(struct sip_msg* msg, int body_delta, int proto)
 				}
 				body_only=0;
 			} /* else
-				DBG("adjust_clen: UDP packet with no clen => "
-						"not adding one \n"); */
+				LM_DBG("UDP packet with no clen => not adding one \n"); */
 		}else{
 			/* Content-Length has been found, remove it */
 			anchor = del_lump(	msg, msg->content_length->body.s - msg->buf,
@@ -2191,8 +2169,8 @@ after_update_via1:
 	}
 	new_buf[new_len]=0;
 	/* copy msg adding/removing lumps */
-	process_lumps(msg, msg->add_rm, new_buf, &offset, &s_offset, send_info);
-	process_lumps(msg, msg->body_lumps, new_buf, &offset, &s_offset,send_info);
+	process_lumps(msg, msg->add_rm, new_buf, &offset, &s_offset, send_info, FLAG_MSG_ALL);
+	process_lumps(msg, msg->body_lumps, new_buf, &offset, &s_offset,send_info, FLAG_MSG_ALL);
 	/* copy the rest of the message */
 	memcpy(new_buf+offset, buf+s_offset, len-s_offset);
 	new_buf[new_len]=0;
@@ -2275,7 +2253,7 @@ char * generate_res_buf_from_sip_res( struct sip_msg* msg,
 	new_len=len+body_delta+lumps_len(msg, msg->add_rm, 0); /*FIXME: we don't
 														know the send sock */
 
-	DBG(" old size: %d, new size: %d\n", len, new_len);
+	LM_DBG("old size: %d, new size: %d\n", len, new_len);
 	new_buf=(char*)pkg_malloc(new_len+1); /* +1 is for debugging
 											 (\0 to print it )*/
 	if (new_buf==0){
@@ -2285,15 +2263,15 @@ char * generate_res_buf_from_sip_res( struct sip_msg* msg,
 	new_buf[new_len]=0; /* debug: print the message */
 	offset=s_offset=0;
 	/*FIXME: no send sock*/
-	process_lumps(msg, msg->add_rm, new_buf, &offset, &s_offset, 0);/*FIXME:*/
-	process_lumps(msg, msg->body_lumps, new_buf, &offset, &s_offset, 0);
+	process_lumps(msg, msg->add_rm, new_buf, &offset, &s_offset, 0, FLAG_MSG_ALL);/*FIXME:*/
+	process_lumps(msg, msg->body_lumps, new_buf, &offset, &s_offset, 0, FLAG_MSG_ALL);
 	/* copy the rest of the message */
 	memcpy(new_buf+offset,
 		buf+s_offset,
 		len-s_offset);
 	 /* send it! */
-	DBG("copied size: orig:%d, new: %d, rest: %d"
-			" msg=\n%s\n", s_offset, offset, len-s_offset, new_buf);
+	LM_DBG("copied size: orig:%d, new: %d, rest: %d msg=\n%s\n",
+			s_offset, offset, len-s_offset, new_buf);
 
 	*returned_len=new_len;
 	return new_buf;
@@ -2889,7 +2867,7 @@ char* create_via_hf( unsigned int *len,
 			return 0; /* we don't need to free anything,
 			                 nothing alloc'ed yet*/
 		}
-		DBG("create_via_hf: id added: <%.*s>, rcv proto=%d\n",
+		LM_DBG("id added: <%.*s>, rcv proto=%d\n",
 				(int)id_len, id_buf, msg->rcv.proto);
 		extra_params.s=id_buf;
 		extra_params.len=id_len;
@@ -2965,7 +2943,7 @@ char * build_only_headers( struct sip_msg* msg, int skip_first_line,
 	offset = 0;
 
 	/* copy message lumps */
-	process_lumps(msg, msg->add_rm, new_buf, &offset, &s_offset, send_info);
+	process_lumps(msg, msg->add_rm, new_buf, &offset, &s_offset, send_info, FLAG_MSG_ALL);
 	/* copy the rest of the message without body */
 	if (len > s_offset) {
 		memcpy(new_buf+offset, buf+s_offset, len-s_offset);
@@ -3015,7 +2993,7 @@ char * build_body( struct sip_msg* msg,
 	offset = 0;
 
 	/* copy body lumps */
-	process_lumps(msg, msg->body_lumps, new_buf, &offset, &s_offset, send_info);
+	process_lumps(msg, msg->body_lumps, new_buf, &offset, &s_offset, send_info, FLAG_MSG_ALL);
 	/* copy the rest of the message without body */
 	if (len > s_offset) {
 		memcpy(new_buf+offset, buf+s_offset, len-s_offset);
@@ -3075,9 +3053,9 @@ char * build_all( struct sip_msg* msg, int touch_clen,
 	offset = s_offset = 0;
 
 	/* copy message lumps */
-	process_lumps(msg, msg->add_rm, new_buf, &offset, &s_offset, send_info);
+	process_lumps(msg, msg->add_rm, new_buf, &offset, &s_offset, send_info, FLAG_MSG_ALL);
 	/* copy body lumps */
-	process_lumps(msg, msg->body_lumps, new_buf, &offset, &s_offset, send_info);
+	process_lumps(msg, msg->body_lumps, new_buf, &offset, &s_offset, send_info, FLAG_MSG_ALL);
 	/* copy the rest of the message */
 	memcpy(new_buf+offset, buf+s_offset, len-s_offset);
 	offset += (len-s_offset);
