@@ -1,43 +1,21 @@
 /*
- * $Id$
- *
  * Copyright (C) 2001-2003 FhG Fokus
  *
- * This file is part of ser, a free SIP server.
+ * This file is part of Kamailio, a free SIP server.
  *
- * ser is free software; you can redistribute it and/or modify
+ * Kamailio is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version
  *
- * For a license to use the ser software under conditions
- * other than those described here, or to purchase support for this
- * software, please contact iptel.org by e-mail at the following addresses:
- *    info@iptel.org
- *
- * ser is distributed in the hope that it will be useful,
+ * Kamailio is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License 
  * along with this program; if not, write to the Free Software 
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- *
- * History:
- * --------
- * 2003-04-04  convenience inbound-uri parser parse_orig_ruri
- *             introduced (jiri)
- * 2003-04-11  new parse_uri introduced (better, parses also some parameters,
- *              works in one pass) (andrei)
- * 2003-04-11  ser_error is now set in parse_uri (andrei)
- * 2003-04-26  ZSW (jiri)
- * 2003-07-03  sips:, r2, lr=on support added (andrei)
- * 2005-02-25  preliminary tel uri support (andrei)
- * 2005-03-03  more tel uri fixes (andrei)
- * 2006-04-20  comp uri param. support (rfc3486) if defined USE_COMP  (andrei)
- * 2007-09-10  introduced phone2tel option which allows NOT to consider
- *             user=phone URIs as TEL URIs
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 
@@ -106,7 +84,9 @@ int parse_uri(char* buf, int len, struct sip_uri* uri)
 					/* tls */
 					      VTLS_L, VTLS_S_FIN,
 					/* sctp */
-					VS_S, VS_C, VS_T, VS_P_FIN
+					VS_S, VS_C, VS_T, VS_P_FIN,
+					/* ws */
+					VW_W, VW_S_FIN
 	};
 	register enum states state;
 	char* s;
@@ -701,6 +681,11 @@ int parse_uri(char* buf, int len, struct sip_uri* uri)
 						v=p;
 						state=VS_S;
 						break;
+					case 'w':
+					case 'W':
+						v=p;
+						state=VW_W;
+						break;
 					default:
 						v=p;
 						state=URI_VAL_P;
@@ -728,6 +713,9 @@ int parse_uri(char* buf, int len, struct sip_uri* uri)
 			value_switch(VS_C, 't', 'T', VS_T);
 			value_switch(VS_T, 'p', 'P', VS_P_FIN);
 			transport_fin(VS_P_FIN, PROTO_SCTP);
+			/* ws */
+			value_switch(VW_W, 's', 'S', VW_S_FIN);
+			transport_fin(VW_S_FIN, PROTO_WS);
 			
 			/* ttl */
 			param_switch(PTTL_T2,  'l', 'L', PTTL_L);
@@ -1094,6 +1082,7 @@ int parse_uri(char* buf, int len, struct sip_uri* uri)
 		case VS_S:
 		case VS_C:
 		case VS_T:
+		case VW_W:
 			uri->params.s=s;
 			uri->params.len=p-s;
 			param_set(b, v);
@@ -1139,6 +1128,12 @@ int parse_uri(char* buf, int len, struct sip_uri* uri)
 			param_set(b, v);
 			uri->proto=PROTO_SCTP;
 			break;
+		case VW_S_FIN:
+			uri->params.s=s;
+			uri->params.len=p-s;
+			param_set(b, v);
+			uri->proto=PROTO_WS;
+			break;
 #ifdef USE_COMP
 		case VCOMP_SIGC_P_FIN:
 			uri->params.s=s;
@@ -1401,6 +1396,7 @@ static str	s_udp  = STR_STATIC_INIT("udp");
 static str	s_tcp  = STR_STATIC_INIT("tcp");
 static str	s_tls  = STR_STATIC_INIT("tls");
 static str	s_sctp = STR_STATIC_INIT("sctp");
+static str	s_ws   = STR_STATIC_INIT("ws");
 
 inline void proto_type_to_str(unsigned short type, str *s) {
 	switch (type) {
@@ -1416,6 +1412,10 @@ inline void proto_type_to_str(unsigned short type, str *s) {
 	case PROTO_SCTP:
 		*s = s_sctp;
 		break;
+	case PROTO_WS:
+	case PROTO_WSS:
+		*s = s_ws;
+		break;
 	default:
 		*s = s_null;
 	}
