@@ -1,26 +1,26 @@
 /*
  * Copyright (C) 2001-2003 FhG Fokus
  *
- * This file is part of SIP-Router, a free SIP server.
+ * This file is part of Kamailio, a free SIP server.
  *
- * SIP-Router is free software; you can redistribute it and/or modify
+ * Kamailio is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version
  *
- * SIP-Router is distributed in the hope that it will be useful,
+ * Kamailio is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License 
  * along with this program; if not, write to the Free Software 
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 /*!
  * \file
- * \brief SIP-router core :: PV API specification
+ * \brief Kamailio core :: PV API specification
  * \ingroup core
  * Module: \ref core
  */
@@ -41,7 +41,6 @@
 #include "pvar.h"
 
 #define PV_TABLE_SIZE	32  /*!< pseudo-variables table size */
-#define PV_CACHE_SIZE	32  /*!< pseudo-variables table size */
 #define TR_TABLE_SIZE	16  /*!< transformations table size */
 
 
@@ -58,14 +57,6 @@ typedef struct _pv_item
 static pv_item_t* _pv_table[PV_TABLE_SIZE];
 static int _pv_table_set = 0;
 
-typedef struct _pv_cache
-{
-	str pvname;
-	unsigned int pvid;
-	pv_spec_t spec;
-	struct _pv_cache *next;
-} pv_cache_t;
-
 static pv_cache_t* _pv_cache[PV_CACHE_SIZE];
 static int _pv_cache_set = 0;
 
@@ -87,6 +78,14 @@ void pv_init_cache(void)
 	_pv_cache_set = 1;
 }
 
+/**
+ *
+ */
+pv_cache_t **pv_cache_get_table(void)
+{
+	if(_pv_cache_set==1) return _pv_cache;
+	return NULL;
+}
 
 /**
  * @brief Check if a char is valid according to the PV syntax
@@ -357,6 +356,70 @@ pv_spec_t* pv_cache_get(str *name)
 	return pv_cache_add(&tname);
 }
 
+str* pv_cache_get_name(pv_spec_t *spec)
+{
+	int i;
+	pv_cache_t *pvi;
+	if(spec==NULL)
+	{
+		LM_ERR("invalid parameters\n");
+		return NULL;
+	}
+
+	if(_pv_cache_set==0)
+		return NULL;
+
+	for(i=0;i<PV_CACHE_SIZE;i++)
+	{
+		pvi = _pv_cache[i];
+		while(pvi)
+		{
+			if(&pvi->spec == spec)
+			{
+				LM_DBG("pvar[%p]->name[%.*s] found in cache\n", spec,
+					pvi->pvname.len, pvi->pvname.s);
+				return &pvi->pvname;
+			}
+			pvi = pvi->next;
+		}
+	}
+	return NULL;
+}
+
+/**
+ *
+ */
+pv_spec_t* pv_spec_lookup(str *name, int *len)
+{
+	pv_spec_t *pvs;
+	str tname;
+
+	if(len!=NULL)
+		*len = 0;
+	if(name->s==NULL || name->len==0)
+	{
+		LM_ERR("invalid parameters\n");
+		return NULL;
+	}
+
+	tname.s = name->s;
+	tname.len = pv_locate_name(name);
+
+	if(tname.len < 0)
+		return NULL;
+
+	if(len!=NULL)
+		*len = tname.len;
+
+	pvs = pv_cache_lookup(&tname);
+
+	if(pvs!=NULL)
+		return pvs;
+
+	LM_DBG("PV <%.*s> is not in cache\n", tname.len, tname.s);
+	return pv_cache_add(&tname);
+}
+
 /**
  *
  */
@@ -460,6 +523,36 @@ int pv_get_strval(struct sip_msg *msg, pv_param_t *param,
 	return 0;
 }
 
+/**
+ * convert strz to pv_value_t
+ */
+int pv_get_strzval(struct sip_msg *msg, pv_param_t *param,
+		pv_value_t *res, char *sval)
+{
+	if(res==NULL)
+		return -1;
+
+	res->rs.s = sval;
+	res->rs.len = strlen(sval);
+	res->flags = PV_VAL_STR;
+	return 0;
+}
+
+/**
+ * convert char* with len to pv_value_t
+ */
+int pv_get_strlval(struct sip_msg *msg, pv_param_t *param,
+		pv_value_t *res, char *sval, int slen)
+{
+	if(res==NULL)
+		return -1;
+
+	res->rs.s = sval;
+	res->rs.len = slen;
+	res->flags = PV_VAL_STR;
+	return 0;
+}
+
 /**
  * convert str-int to pv_value_t (type is str)
  */
@@ -499,6 +592,9 @@ static int pv_get_marker(struct sip_msg *msg, pv_param_t *param,
 			(int)pv_str_marker.s[0]);
 }
 
+static char pv_str_empty_buf[2];
+static char pv_str_null_buf[8];
+
 static str pv_str_empty  = { "", 0 };
 static str pv_str_null   = { "<null>", 6 };
 int pv_get_null(struct sip_msg *msg, pv_param_t *param, pv_value_t *res)
@@ -512,6 +608,21 @@ int pv_get_null(struct sip_msg *msg, pv_param_t *param, pv_value_t *res)
 	return 0;
 }
 
+/**
+ *
+ */
+int pv_get_strempty(struct sip_msg *msg, pv_param_t *param, pv_value_t *res)
+{
+	if(res==NULL)
+		return -1;
+
+	res->rs = pv_str_empty;
+	res->ri = 0;
+	res->flags = PV_VAL_STR;
+	return 0;
+}
+
+
 /**
  *
  */
@@ -589,6 +700,11 @@ int pv_parse_index(pv_spec_p sp, str *in)
 		sp->pvp.pvi.type = PV_IDX_ALL;
 		return 0;
 	}
+	if(*p=='+' && in->len==1)
+	{
+		sp->pvp.pvi.type = PV_IDX_ITR;
+		return 0;
+	}
 	sign = 1;
 	if(*p=='-')
 	{
@@ -932,6 +1048,7 @@ int pv_parse_format(str *in, pv_elem_p *el)
 	int n = 0;
 	pv_elem_p e, e0;
 	str s;
+	int len;
 
 	if(in==NULL || in->s==NULL || el==NULL)
 		return -1;
@@ -974,7 +1091,10 @@ int pv_parse_format(str *in, pv_elem_p *el)
 			break;
 		s.s = p;
 		s.len = in->s+in->len-p;
-		p0 = pv_parse_spec(&s, &e->spec);
+		e->spec = pv_spec_lookup(&s, &len);
+		if(e->spec==NULL)
+			goto error;
+		p0 = p + len;
 		
 		if(p0==NULL)
 			goto error;
@@ -1143,7 +1263,10 @@ int pv_get_spec_index(struct sip_msg* msg, pv_param_p ip, int *idx, int *flags)
 		*flags = PV_IDX_ALL;
 		return 0;
 	}
-	
+	if(ip->pvi.type == PV_IDX_ITR) {
+		*flags = PV_IDX_ITR;
+		return 0;
+	}
 	if(ip->pvi.type == PV_IDX_INT)
 	{
 		*idx = ip->pvi.u.ival;
@@ -1239,8 +1362,8 @@ int pv_printf(struct sip_msg* msg, pv_elem_p list, char *buf, int *len)
 			}
 		}
 		/* put the value of the specifier */
-		if(it->spec.type!=PVT_NONE
-				&& pv_get_spec_value(msg, &(it->spec), &tok)==0)
+		if(it->spec!=NULL && it->spec->type!=PVT_NONE
+				&& pv_get_spec_value(msg, it->spec, &tok)==0)
 		{
 			if(tok.flags&PV_VAL_NULL)
 				tok.rs = pv_str_null;
@@ -1391,8 +1514,10 @@ int pv_elem_free_all(pv_elem_p log)
 void pv_value_destroy(pv_value_t *val)
 {
 	if(val==0) return;
+
 	if(val->flags&PV_VAL_PKG) pkg_free(val->rs.s);
-	if(val->flags&PV_VAL_SHM) shm_free(val->rs.s);
+	else if(val->flags&PV_VAL_SHM) shm_free(val->rs.s);
+
 	memset(val, 0, sizeof(pv_value_t));
 }
 
@@ -1783,6 +1908,13 @@ int pv_init_api(void)
 	tr_init_table();
 	if(pv_init_buffer()<0)
 		return -1;
+
+	pv_str_empty_buf[0] = '\0';
+	pv_str_empty_buf[1] = '\0';
+	pv_str_empty.s = pv_str_empty_buf;
+	strcpy(pv_str_null_buf, "<null>");
+	pv_str_null.s = pv_str_null_buf;
+
 	if(register_pvars_mod("core", _core_pvs)<0)
 		return -1;
 	return 0;
@@ -1803,11 +1935,13 @@ void pv_destroy_api(void)
  * - buffer to print PVs
  */
 static char **_pv_print_buffer = NULL;
-#define PV_DEFAULT_PRINT_BUFFER_SIZE 1024
+#define PV_DEFAULT_PRINT_BUFFER_SIZE 8192 /* 8kB */
 static int _pv_print_buffer_size  = PV_DEFAULT_PRINT_BUFFER_SIZE;
+static int _pv_print_buffer_size_active  = 0;
 /* 6 mod params + 4 direct usage from mods */
 #define PV_DEFAULT_PRINT_BUFFER_SLOTS 10
 static int _pv_print_buffer_slots = PV_DEFAULT_PRINT_BUFFER_SLOTS;
+static int _pv_print_buffer_slots_active = 0;
 static int _pv_print_buffer_index = 0;
 
 /**
@@ -1841,6 +1975,9 @@ int pv_init_buffer(void)
 	}
 	LM_DBG("PV print buffer initialized to [%d][%d]\n",
 			_pv_print_buffer_slots, _pv_print_buffer_size);
+	_pv_print_buffer_slots_active = _pv_print_buffer_slots;
+	_pv_print_buffer_size_active = _pv_print_buffer_size;
+
 	return 0;
 }
 
@@ -1853,12 +1990,14 @@ void pv_destroy_buffer(void)
 
 	if(_pv_print_buffer==NULL)
 		return;
-	for(i=0; i<_pv_print_buffer_slots; i++)
+	for(i=0; i<_pv_print_buffer_slots_active; i++)
 	{
 		if(_pv_print_buffer[i]!=NULL)
 			pkg_free(_pv_print_buffer[i]);
 	}
 	pkg_free(_pv_print_buffer);
+	_pv_print_buffer_slots_active = 0;
+	_pv_print_buffer_size_active = 0;
 	_pv_print_buffer = NULL;
 }
 
@@ -1867,8 +2006,8 @@ void pv_destroy_buffer(void)
  */
 int pv_reinit_buffer(void)
 {
-	if(_pv_print_buffer_size==PV_DEFAULT_PRINT_BUFFER_SIZE
-			&& _pv_print_buffer_slots==PV_DEFAULT_PRINT_BUFFER_SLOTS)
+	if(_pv_print_buffer_size==_pv_print_buffer_size_active
+			&& _pv_print_buffer_slots==_pv_print_buffer_slots_active)
 		return 0;
 	pv_destroy_buffer();
 	return pv_init_buffer();
