@@ -1,51 +1,32 @@
 
-/* $Id$
- *
+/* 
  * find & manage listen addresses 
  *
  * Copyright (C) 2001-2003 FhG Fokus
  *
- * This file is part of ser, a free SIP server.
+ * This file is part of Kamailio, a free SIP server.
  *
- * ser is free software; you can redistribute it and/or modify
+ * Kamailio is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version
  *
- * For a license to use the ser software under conditions
- * other than those described here, or to purchase support for this
- * software, please contact iptel.org by e-mail at the following addresses:
- *    info@iptel.org
- *
- * ser is distributed in the hope that it will be useful,
+ * Kamailio is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License 
  * along with this program; if not, write to the Free Software 
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
-/*
- * This file contains code that initializes and handles ser listen addresses
- * lists (struct socket_info). It is used mainly on startup.
- * 
- * History:
- * --------
- *  2003-10-22  created by andrei
- *  2004-10-10  added grep_sock_info (andrei)
- *  2004-11-08  added find_si (andrei)
- *  2007-08-23  added detection for INADDR_ANY types of sockets (andrei)
- *  2008-08-08  sctp support (andrei)
- *  2008-08-15  support for handling sctp multihomed sockets (andrei)
- *  2008-10-15  fixed protocol list iteration when some protocols are
- *               compile time disabled (andrei)
- */
-
 
 /*!
  * \file
- * \brief SIP-router core :: 
+ * \brief Kamailio core :: find & manage listen addresses 
+ *
+ * This file contains code that initializes and handles Kamailio listen addresses
+ * lists (struct socket_info). It is used mainly on startup.
  * \ingroup core
  * Module: \ref core
  */
@@ -60,6 +41,7 @@
 
 #include <sys/ioctl.h>
 #include <net/if.h>
+#include <ifaddrs.h>
 #ifdef HAVE_SYS_SOCKIO_H
 #include <sys/sockio.h>
 #endif
@@ -140,6 +122,8 @@ inline static void addr_info_list_ins_lst(struct addr_info* lst,
 /* protocol order, filled by init_proto_order() */
 enum sip_protos nxt_proto[PROTO_LAST+1]=
 { PROTO_UDP, PROTO_TCP, PROTO_TLS, PROTO_SCTP, 0 };
+/* Deliberately left PROTO_WS and PROTO_WSS out of this as they are just
+   upgraded TCP and TLS connections */
 
 
 
@@ -157,7 +141,7 @@ static int init_addr_info(struct addr_info* a,
 	a->flags=flags;
 	return 0;
 error:
-	LOG(L_ERR, "ERROR: init_addr_info: memory allocation error\n");
+	LM_ERR("memory allocation error\n");
 	return -1;
 }
 
@@ -176,7 +160,7 @@ static inline struct addr_info* new_addr_info(char* name,
 	if (init_addr_info(al, name, gf)!=0) goto error;
 	return al;
 error:
-	LOG(L_ERR, "ERROR: new_addr_info: memory allocation error\n");
+	LM_ERR("memory allocation error\n");
 	if (al){
 		if (al->name.s) pkg_free(al->name.s);
 		pkg_free(al);
@@ -258,7 +242,7 @@ static inline struct socket_info* new_sock_info(	char* name,
 	si->addr_info_lst=0;
 	for (n=addr_l; n; n=n->next){
 		if (new_addr_info2list(n->name, n->flags, &si->addr_info_lst)!=0){
-			LOG(L_ERR, "ERROR: new_sockk_info:new_addr_info2list failed\n");
+			LM_ERR("new_addr_info2list failed\n");
 			goto error;
 		}
 	}
@@ -294,14 +278,14 @@ static inline struct socket_info* new_sock_info(	char* name,
 
 		he=resolvehost(si->useinfo.name.s);
 		if (he==0){
-			LM_ERR(" unable to resolve advertised name %s\n", si->useinfo.name.s);
+			LM_ERR("unable to resolve advertised name %s\n", si->useinfo.name.s);
 			goto error;
 		}
 		hostent2ip_addr(&si->useinfo.address, he, 0);
 	}
 	return si;
 error:
-	LOG(L_ERR, "ERROR: new_sock_info: memory allocation error\n");
+	LM_ERR("memory allocation error\n");
 	if (si) {
 		if(si->name.s)
 			pkg_free(si->name.s);
@@ -329,7 +313,7 @@ static void free_sock_info(struct socket_info* si)
 
 
 
-static char* get_valid_proto_name(unsigned short proto)
+char* get_valid_proto_name(unsigned short proto)
 {
 	switch(proto){
 		case PROTO_NONE:
@@ -435,7 +419,7 @@ static int fix_sock_str(struct socket_info* si)
 	
 	si->sock_str.s = pkg_malloc(len + 1);
 	if (si->sock_str.s == NULL) {
-		ERR("fix_sock_str: No memory left\n");
+		LM_ERR("No memory left\n");
 		return -1;
 	}
 	if (socketinfo2str(si->sock_str.s, &len, si, 0) < 0) {
@@ -452,7 +436,7 @@ static int fix_sock_str(struct socket_info* si)
 
 		si->useinfo.sock_str.s = pkg_malloc(len + 1);
 		if (si->useinfo.sock_str.s == NULL) {
-			ERR("fix_sock_str: No memory left\n");
+			LM_ERR("No memory left\n");
 			return -1;
 		}
 		if (socketinfo2str(si->useinfo.sock_str.s, &len, si, 1) < 0) {
@@ -476,11 +460,13 @@ struct socket_info** get_sock_info_list(unsigned short proto)
 			return &udp_listen;
 			break;
 		case PROTO_TCP:
+		case PROTO_WS:
 #ifdef USE_TCP
 			return &tcp_listen;
 #endif
 			break;
 		case PROTO_TLS:
+		case PROTO_WSS:
 #ifdef USE_TLS
 			return &tls_listen;
 #endif
@@ -491,7 +477,7 @@ struct socket_info** get_sock_info_list(unsigned short proto)
 #endif
 			break;
 		default:
-			LOG(L_CRIT, "BUG: get_sock_info_list: invalid proto %d\n", proto);
+			LM_CRIT("invalid proto %d\n", proto);
 	}
 	return 0;
 }
@@ -509,9 +495,7 @@ struct socket_info** get_sock_info_list(unsigned short proto)
 inline static int si_hname_cmp(str* host, str* name, str* addr_str, 
 								struct ip_addr* ip_addr, int flags)
 {
-#ifdef USE_IPV6
 	struct ip_addr* ip6;
-#endif
 	
 	if ( (host->len==name->len) && 
 		(strncasecmp(host->s, name->s, name->len)==0) /*slower*/)
@@ -520,7 +504,6 @@ inline static int si_hname_cmp(str* host, str* name, str* addr_str,
 		 * ipv6 addresses if we are lucky*/
 		goto found;
 	/* check if host == ip address */
-#ifdef USE_IPV6
 	/* ipv6 case is uglier, host can be [3ffe::1] */
 	ip6=str2ip6(host);
 	if (ip6){
@@ -530,7 +513,6 @@ inline static int si_hname_cmp(str* host, str* name, str* addr_str,
 			return -1; /* no match, but this is an ipv6 address
 						 so no point in trying ipv4 */
 	}
-#endif
 	/* ipv4 */
 	if ( (!(flags&SI_IS_IP)) && (host->len==addr_str->len) && 
 			(memcmp(host->s, addr_str->s, addr_str->len)==0) )
@@ -559,14 +541,14 @@ struct socket_info* grep_sock_info(str* host, unsigned short port,
 	unsigned short c_proto;
 	
 	hname=*host;
-#ifdef USE_IPV6
 	if ((hname.len>2)&&((*hname.s)=='[')&&(hname.s[hname.len-1]==']')){
 		/* ipv6 reference, skip [] */
 		hname.s++;
 		hname.len-=2;
 	}
-#endif
+
 	c_proto=(proto!=PROTO_NONE)?proto:PROTO_UDP;
+retry:
 	do{
 		/* get the proper sock_list */
 		list=get_sock_info_list(c_proto);
@@ -574,16 +556,14 @@ struct socket_info* grep_sock_info(str* host, unsigned short port,
 		if (list==0) /* disabled or unknown protocol */
 			continue;
 		for (si=*list; si; si=si->next){
-			DBG("grep_sock_info - checking if host==us: %d==%d &&"
-					" [%.*s] == [%.*s]\n",
+			LM_DBG("checking if host==us: %d==%d && [%.*s] == [%.*s]\n",
 						hname.len,
 						si->name.len,
 						hname.len, hname.s,
 						si->name.len, si->name.s
 				);
 			if (port) {
-				DBG("grep_sock_info - checking if port %d (advertise %d)"
-						" matches port %d\n",
+				LM_DBG("checking if port %d (advertise %d) matches port %d\n",
 						si->port_no, si->useinfo.port_no, port);
 				if (si->port_no!=port && si->useinfo.port_no!=port) {
 					continue;
@@ -594,7 +574,7 @@ struct socket_info* grep_sock_info(str* host, unsigned short port,
 				goto found;
 			if(si->useinfo.name.s!=NULL)
 			{
-				DBG("grep_sock_info - checking advertise if host==us:"
+				LM_DBG("checking advertise if host==us:"
 						" %d==%d && [%.*s] == [%.*s]\n",
 						hname.len,
 						si->useinfo.name.len,
@@ -612,7 +592,15 @@ struct socket_info* grep_sock_info(str* host, unsigned short port,
 									&ai->address, ai->flags)==0)
 					goto found;
 		}
+
 	}while( (proto==0) && (c_proto=next_proto(c_proto)) );
+
+#ifdef USE_TLS
+	if (unlikely(c_proto == PROTO_WS)) {
+		c_proto = PROTO_WSS;
+		goto retry;
+	}
+#endif
 /* not_found: */
 	return 0;
 found:
@@ -643,8 +631,7 @@ struct socket_info* grep_sock_info_by_port(unsigned short port,
 			continue;
 		
 		for (si=*list; si; si=si->next){
-			DBG("grep_sock_info_by_port - checking if port %d matches"
-					" port %d\n", si->port_no, port);
+			LM_DBG("checking if port %d matches port %d\n", si->port_no, port);
 			if (si->port_no==port) {
 				goto found;
 			}
@@ -718,7 +705,7 @@ static struct socket_info* new_sock2list(char* name, struct name_lst* addr_l,
 	
 	si=new_sock_info(name, addr_l, port, proto, usename, useport, flags);
 	if (si==0){
-		LOG(L_ERR, "ERROR: new_sock2list: new_sock_info failed\n");
+		LM_ERR("new_sock_info failed\n");
 		goto error;
 	}
 	if(socket_workers>0) {
@@ -748,7 +735,7 @@ static struct socket_info* new_sock2list_after(char* name,
 	
 	si=new_sock_info(name, addr_l, port, proto, usename, useport, flags);
 	if (si==0){
-		LOG(L_ERR, "ERROR: new_sock2list_after: new_sock_info failed\n");
+		LM_ERR("new_sock_info failed\n");
 		goto error;
 	}
 	sock_listins(si, after);
@@ -796,7 +783,7 @@ int add_listen_advertise_iface(char* name, struct name_lst* addr_l,
 		if (c_proto!=PROTO_SCTP){
 			if (new_sock2list(name, 0, c_port, c_proto, usename, useport,
 								flags & ~SI_IS_MHOMED, list)==0){
-				LOG(L_ERR, "ERROR: add_listen_iface: new_sock2list failed\n");
+				LM_ERR("new_sock2list failed\n");
 				goto error;
 			}
 			/* add the other addresses in the list as separate sockets
@@ -805,15 +792,14 @@ int add_listen_advertise_iface(char* name, struct name_lst* addr_l,
 				if (new_sock2list(a_l->name, 0, c_port, 
 									c_proto, usename, useport,
 									flags & ~SI_IS_MHOMED, list)==0){
-					LOG(L_ERR, "ERROR: add_listen_iface: new_sock2list"
-								" failed\n");
+					LM_ERR("new_sock2list failed\n");
 					goto error;
 				}
 			}
 		}else{
 			if (new_sock2list(name, addr_l, c_port, c_proto, usename, useport,
 						flags, list)==0){
-				LOG(L_ERR, "ERROR: add_listen_iface: new_sock2list failed\n");
+				LM_ERR("new_sock2list failed\n");
 				goto error;
 			}
 		}
@@ -833,6 +819,7 @@ int add_listen_iface(char* name, struct name_lst* addr_l,
 }
 #ifdef __OS_linux
 
+#include "linux/types.h"
 #include "linux/netlink.h"
 #include "linux/rtnetlink.h"
 #include "arpa/inet.h"
@@ -1143,7 +1130,8 @@ error:
 	return -1;
 
 }
-/* add all family type addresses of interface if_to the socket_info array
+/* add all family type addresses of interface if_name to the socket_info array
+ * if family ==0, uses all families
  * if if_name==0, adds all addresses on all interfaces
  * uses RTNETLINK sockets to get addresses on the present interface on LINUX
  * return: -1 on error, 0 on success
@@ -1171,9 +1159,10 @@ int add_interfaces_via_netlink(char* if_name, int family, unsigned short port,
 			//if(! (ifaces[i].flags & IFF_UP) ) continue;
 
 			for(tmp = ifaces[i].addresses; tmp; tmp = tmp->next){
-				LM_DBG("\t in add_iface_via_netlink Name %s Address %s\n", ifaces[i].name, tmp->addr);
-		                /* match family */
-                                if (family == tmp->family){
+				LM_DBG("in add_iface_via_netlink Name %s Address %s\n",
+							ifaces[i].name, tmp->addr);
+					/* match family */
+					if (family && family == tmp->family){
 					/* check if loopback */
 					if (ifaces[i].flags & IFF_LOOPBACK){
 						LM_DBG("INTERFACE %s is loopback", ifaces[i].name);
@@ -1181,8 +1170,7 @@ int add_interfaces_via_netlink(char* if_name, int family, unsigned short port,
 					}
 					/* save the info */
 					if (new_addr_info2list(tmp->addr, flags, ai_l)!=0){
-						LOG(L_ERR, "ERROR: add_interfaces: "
-							"new_addr_info2list failed\n");
+						LM_ERR("new_addr_info2list failed\n");
 						goto error;
 			    		}
 				}
@@ -1196,135 +1184,51 @@ error:
 #endif /* __OS_linux */
 
 /* add all family type addresses of interface if_name to the socket_info array
+ * if family ==0, uses all families
  * if if_name==0, adds all addresses on all interfaces
- * WARNING: it only works with ipv6 addresses on FreeBSD
  * return: -1 on error, 0 on success
  */
 int add_interfaces(char* if_name, int family, unsigned short port,
 					unsigned short proto,
 					struct addr_info** ai_l)
 {
-	struct ifconf ifc;
-	struct ifreq ifr;
-	struct ifreq ifrcopy;
-	char*  last;
-	char* p;
-	int size;
-	int lastlen;
-	int s;
 	char* tmp;
 	struct ip_addr addr;
-	int ret;
+	int ret = -1;
 	enum si_flags flags;
+	struct ifaddrs *ifap, *ifa;
 
-#ifdef HAVE_SOCKADDR_SA_LEN
-	#ifndef MAX
-		#define MAX(a,b) ( ((a)>(b))?(a):(b))
-	#endif
-#endif
-	/* ipv4 or ipv6 only*/
-	flags=SI_NONE;
-	s=socket(family, SOCK_DGRAM, 0);
-	ret=-1;
-	lastlen=0;
-	ifc.ifc_req=0;
-	for (size=100; ; size*=2){
-		ifc.ifc_len=size*sizeof(struct ifreq);
-		ifc.ifc_req=(struct ifreq*) pkg_malloc(size*sizeof(struct ifreq));
-		if (ifc.ifc_req==0){
-			LOG(L_ERR, "ERROR: add_interfaces: memory allocation failure\n");
-			goto error;
-		}
-		if (ioctl(s, SIOCGIFCONF, &ifc)==-1){
-			if(errno==EBADF) return 0; /* invalid descriptor => no such ifs*/
-			LOG(L_ERR, "ERROR: add_interfaces: ioctl failed: %s\n",
-					strerror(errno));
-			goto error;
-		}
-		if  ((lastlen) && (ifc.ifc_len==lastlen)) break; /*success,
-														   len not changed*/
-		lastlen=ifc.ifc_len;
-		/* try a bigger array*/
-		pkg_free(ifc.ifc_req);
-	}
-	
-	last=(char*)ifc.ifc_req+ifc.ifc_len;
-	for(p=(char*)ifc.ifc_req; p<last;
-			p+=
-			#ifdef __OS_linux
-				sizeof(ifr) /* works on x86_64 too */
-			#else
-				(sizeof(ifr.ifr_name)+
-				#ifdef  HAVE_SOCKADDR_SA_LEN
-					MAX(ifr.ifr_addr.sa_len, sizeof(struct sockaddr))
-				#else
-					( (ifr.ifr_addr.sa_family==AF_INET)?
-						sizeof(struct sockaddr_in):
-					#ifdef USE_IPV6
-						((ifr.ifr_addr.sa_family==AF_INET6)?
-						sizeof(struct sockaddr_in6):sizeof(struct sockaddr)) )
-					#else /* USE_IPV6 */
-						sizeof(struct sockaddr) )
-					#endif /* USE_IPV6 */
-				#endif
-				)
-			#endif
-		)
+	if (getifaddrs (&ifap) != 0) {
+		LM_ERR("getifaddrs failed\n");
+		return -1;
+        }
+
+	for (ifa = ifap; ifa; ifa = ifa->ifa_next)
 	{
-		/* copy contents into ifr structure
-		 * warning: it might be longer (e.g. ipv6 address) */
-		memcpy(&ifr, p, sizeof(ifr));
-		if (ifr.ifr_addr.sa_family!=family){
-			/*printf("strange family %d skipping...\n",
-					ifr->ifr_addr.sa_family);*/
+		if (if_name && strcmp(if_name, ifa->ifa_name))
 			continue;
+		if (family && family != ifa->ifa_addr->sa_family)
+			continue;
+		sockaddr2ip_addr(&addr, (struct sockaddr*)ifa->ifa_addr);
+		tmp=ip_addr2a(&addr);
+		if (ifa->ifa_flags & IFF_LOOPBACK) 
+			flags = SI_IS_LO;
+		else
+			flags = SI_NONE;
+		if (new_addr_info2list(tmp, flags, ai_l)!=0)
+		{
+			LM_ERR("new_addr_info2list failed\n");
+			ret = -1;
+			break;
 		}
-		
-		/*get flags*/
-		ifrcopy=ifr;
-		if (ioctl(s, SIOCGIFFLAGS,  &ifrcopy)!=-1){ /* ignore errors */
-			/* ignore down ifs only if listening on all of them*/
-			if (if_name==0){ 
-				/* if if not up, skip it*/
-				if (!(ifrcopy.ifr_flags & IFF_UP)) continue;
-			}
-		}
-		
-		
-		
-		if ((if_name==0)||
-			(strncmp(if_name, ifr.ifr_name, sizeof(ifr.ifr_name))==0)){
-			
-			/*add address*/
-			sockaddr2ip_addr(&addr, 
-					(struct sockaddr*)(p+(long)&((struct ifreq*)0)->ifr_addr));
-			if ((tmp=ip_addr2a(&addr))==0) goto error;
-			/* check if loopback */
-			if (ifrcopy.ifr_flags & IFF_LOOPBACK) 
-				flags|=SI_IS_LO;
-			/* save the info */
-			if (new_addr_info2list(tmp, flags, ai_l)!=0){
-				LOG(L_ERR, "ERROR: add_interfaces: "
-						"new_addr_info2list failed\n");
-				goto error;
-			}
-			ret=0;
-		}
-			/*
-			printf("%s:\n", ifr->ifr_name);
-			printf("        ");
-			print_sockaddr(&(ifr->ifr_addr));
-			printf("        ");
-			ls_ifflags(ifr->ifr_name, family, options);
-			printf("\n");*/
+		LM_DBG("If: %8s Fam: %8x Flg: %16lx Adr: %s\n",
+				ifa->ifa_name, ifa->ifa_addr->sa_family,
+				(unsigned long)ifa->ifa_flags, tmp);
+
+		ret = 0;
 	}
-	pkg_free(ifc.ifc_req); /*clean up*/
-	close(s);
+	freeifaddrs(ifap);
 	return  ret;
-error:
-	if (ifc.ifc_req) pkg_free(ifc.ifc_req);
-	close(s);
-	return -1;
 }
 
 
@@ -1345,20 +1249,20 @@ static int fix_hostname(str* name, struct ip_addr* address, str* address_str,
 	/* get "official hostnames", all the aliases etc. */
 	he=resolvehost(name->s);
 	if (he==0){
-		LOG(L_ERR, "ERROR: fix_hostname: could not resolve %s\n", name->s);
+		LM_ERR("could not resolve %s\n", name->s);
 		goto error;
 	}
 	/* check if we got the official name */
 	if (strcasecmp(he->h_name, name->s)!=0){
 		if (sr_auto_aliases && 
 				add_alias(name->s, name->len, s->port_no, s->proto)<0){
-			LOG(L_ERR, "ERROR: fix_hostname: add_alias failed\n");
+			LM_ERR("add_alias failed\n");
 		}
 		/* change the official name */
 		pkg_free(name->s);
 		name->s=(char*)pkg_malloc(strlen(he->h_name)+1);
 		if (name->s==0){
-			LOG(L_ERR,  "ERROR: fix_hostname: out of memory.\n");
+			LM_ERR("out of memory.\n");
 			goto error;
 		}
 		name->len=strlen(he->h_name);
@@ -1367,7 +1271,7 @@ static int fix_hostname(str* name, struct ip_addr* address, str* address_str,
 	/* add the aliases*/
 	for(h=he->h_aliases; sr_auto_aliases && h && *h; h++)
 		if (add_alias(*h, strlen(*h), s->port_no, s->proto)<0){
-			LOG(L_ERR, "ERROR: fix_hostname: add_alias failed\n");
+			LM_ERR("add_alias failed\n");
 		}
 	hostent2ip_addr(address, he, 0); /*convert to ip_addr format*/
 	if (type_flags){
@@ -1376,7 +1280,7 @@ static int fix_hostname(str* name, struct ip_addr* address, str* address_str,
 	if ((tmp=ip_addr2a(address))==0) goto error;
 	address_str->s=pkg_malloc(strlen(tmp)+1);
 	if (address_str->s==0){
-		LOG(L_ERR, "ERROR: fix_hostname: out of memory.\n");
+		LM_ERR("out of memory.\n");
 		goto error;
 	}
 	strncpy(address_str->s, tmp, strlen(tmp)+1);
@@ -1388,17 +1292,16 @@ static int fix_hostname(str* name, struct ip_addr* address, str* address_str,
 		/* do rev. DNS on it (for aliases)*/
 		he=rev_resolvehost(address);
 		if (he==0){
-			LOG(L_WARN, "WARNING: fix_hostname: could not rev. resolve %s\n",
-					name->s);
+			LM_WARN("could not rev. resolve %s\n", name->s);
 		}else{
 			/* add the aliases*/
 			if (add_alias(he->h_name, strlen(he->h_name), s->port_no,
 							s->proto)<0){
-				LOG(L_ERR, "ERROR: fix_hostname: add_alias failed\n");
+				LM_ERR("add_alias failed\n");
 			}
 			for(h=he->h_aliases; h && *h; h++)
 				if (add_alias(*h, strlen(*h), s->port_no, s->proto) < 0){
-					LOG(L_ERR, "ERROR: fix_hostname: add_alias failed\n");
+					LM_ERR("add_alias failed\n");
 				}
 		}
 	}
@@ -1495,7 +1398,7 @@ static int fix_socket_list(struct socket_info **list, int* type_flags)
 	for (si=*list;si;){
 		next=si->next;
 		ai_lst=0;
-		if (add_interfaces(si->name.s, AF_INET, si->port_no,
+		if (add_interfaces(si->name.s, 0, si->port_no,
 							si->proto, &ai_lst)!=-1){
 			if (si->flags & SI_IS_MHOMED){
 				if((new_si=new_sock2list_after(ai_lst->name.s, 0, si->port_no,
@@ -1561,7 +1464,7 @@ static int fix_socket_list(struct socket_info **list, int* type_flags)
 	}
 	/* get ips & fill the port numbers*/
 #ifdef EXTRA_DEBUG
-	DBG("Listening on \n");
+	LM_DBG("Listening on\n");
 #endif
 	for (si=*list;si;si=si->next){
 		/* fix port number, port_no should be !=0 here */
@@ -1574,13 +1477,12 @@ static int fix_socket_list(struct socket_info **list, int* type_flags)
 		}
 		tmp=int2str(si->port_no, &len);
 		if (len>=MAX_PORT_LEN){
-			LOG(L_ERR, "ERROR: fix_socket_list: bad port number: %d\n", 
-						si->port_no);
+			LM_ERR("bad port number: %d\n", si->port_no);
 			goto error;
 		}
 		si->port_no_str.s=(char*)pkg_malloc(len+1);
 		if (si->port_no_str.s==0){
-			LOG(L_ERR, "ERROR: fix_socket_list: out of memory.\n");
+			LM_ERR("out of memory.\n");
 			goto error;
 		}
 		strncpy(si->port_no_str.s, tmp, len+1);
@@ -1767,7 +1669,7 @@ static int fix_socket_list(struct socket_info **list, int* type_flags)
 #endif
 			){
 			if (si->flags & SI_IS_MCAST){
-				LOG(L_WARN, "WARNING: removing entry %s:%s [%s]:%s\n",
+				LM_WARN("removing entry %s:%s [%s]:%s\n",
 					get_valid_proto_name(si->proto), si->name.s, 
 					si->address_str.s, si->port_no_str.s);
 				l = si;
@@ -1778,7 +1680,7 @@ static int fix_socket_list(struct socket_info **list, int* type_flags)
 				ail=si->addr_info_lst;
 				while(ail){
 					if (ail->flags & SI_IS_MCAST){
-						LOG(L_WARN, "WARNING: removing mh entry %s:%s"
+						LM_WARN("removing mh entry %s:%s"
 								" [%s]:%s\n",
 								get_valid_proto_name(si->proto), ail->name.s, 
 								ail->address_str.s, si->port_no_str.s);
@@ -1828,13 +1730,11 @@ int fix_all_socket_lists()
 		){
 		/* get all listening ipv4/ipv6 interfaces */
 		if ( ( (add_interfaces(0, AF_INET, 0,  PROTO_UDP, &ai_lst)==0)
-#ifdef USE_IPV6
 #ifdef __OS_linux
 		&&  (!auto_bind_ipv6 || add_interfaces_via_netlink(0, AF_INET6, 0, PROTO_UDP, &ai_lst) == 0)
 #else
-		&& ( !auto_bind_ipv6 || add_interfaces(0, AF_INET6, 0,  PROTO_UDP, &ai_lst) !=0 ) /* add_interface does not work for IPv6 on Linux */
+		&& ( !auto_bind_ipv6 || add_interfaces(0, AF_INET6, 0,  PROTO_UDP, &ai_lst) ==0 ) /* add_interface does not work for IPv6 on Linux */
 #endif /* __OS_linux */
-#endif /* USE_IPV6 */
 			 ) && (addr_info_to_si_lst(ai_lst, 0, PROTO_UDP, 0, &udp_listen)==0)){
 			free_addr_info_lst(&ai_lst);
 			ai_lst=0;
@@ -1842,13 +1742,11 @@ int fix_all_socket_lists()
 #ifdef USE_TCP
 			if (!tcp_disable){
 				if ( ((add_interfaces(0, AF_INET, 0,  PROTO_TCP, &ai_lst)!=0)
-#ifdef USE_IPV6
 #ifdef __OS_linux
     				|| (auto_bind_ipv6 && add_interfaces_via_netlink(0, AF_INET6, 0, PROTO_TCP, &ai_lst) != 0)
 #else
 				|| (auto_bind_ipv6 && add_interfaces(0, AF_INET6, 0,  PROTO_TCP, &ai_lst) !=0 )
 #endif /* __OS_linux */
-#endif /* USE_IPV6 */
 				) || (addr_info_to_si_lst(ai_lst, 0, PROTO_TCP, 0,
 										 				&tcp_listen)!=0))
 					goto error;
@@ -1858,13 +1756,11 @@ int fix_all_socket_lists()
 				if (!tls_disable){
 					if (((add_interfaces(0, AF_INET, 0, PROTO_TLS,
 										&ai_lst)!=0)
-#ifdef USE_IPV6
 #ifdef __OS_linux
     				|| (auto_bind_ipv6 && add_interfaces_via_netlink(0, AF_INET6, 0, PROTO_TLS, &ai_lst) != 0)
 #else
 				|| (auto_bind_ipv6 && add_interfaces(0, AF_INET6, 0,  PROTO_TLS, &ai_lst)!=0)
 #endif /* __OS_linux */
-#endif /* USE_IPV6 */
 					) || (addr_info_to_si_lst(ai_lst, 0, PROTO_TLS, 0,
 										 				&tls_listen)!=0))
 						goto error;
@@ -1877,13 +1773,11 @@ int fix_all_socket_lists()
 #ifdef USE_SCTP
 			if (!sctp_disable){
 				if (((add_interfaces(0, AF_INET, 0,  PROTO_SCTP, &ai_lst)!=0)
-#ifdef USE_IPV6
 #ifdef __OS_linux
     				|| (auto_bind_ipv6 && add_interfaces_via_netlink(0, AF_INET6, 0, PROTO_SCTP, &ai_lst) != 0)
 #else
 				|| (auto_bind_ipv6 && add_interfaces(0, AF_INET6, 0,  PROTO_SCTP, &ai_lst) != 0)
 #endif /* __OS_linux */
-#endif /* USE_IPV6 */
 					) || (addr_info_to_si_lst(ai_lst, 0, PROTO_SCTP, 0,
 							 				&sctp_listen)!=0))
 					goto error;
@@ -1895,21 +1789,18 @@ int fix_all_socket_lists()
 			/* if error fall back to get hostname */
 			/* get our address, only the first one */
 			if (uname (&myname) <0){
-				LOG(L_ERR, "ERROR: fix_all_socket_lists: cannot determine"
-						" hostname, try -l address\n");
+				LM_ERR("cannot determine hostname, try -l address\n");
 				goto error;
 			}
 			if (add_listen_iface(myname.nodename, 0, 0, 0, 0)!=0){
-				LOG(L_ERR, "ERROR: fix_all_socket_lists: add_listen_iface "
-						"failed \n");
+				LM_ERR("add_listen_iface failed \n");
 				goto error;
 			}
 		}
 	}
 	flags=0;
 	if (fix_socket_list(&udp_listen, &flags)!=0){
-		LOG(L_ERR, "ERROR: fix_all_socket_lists: fix_socket_list"
-				" udp failed\n");
+		LM_ERR("fix_socket_list udp failed\n");
 		goto error;
 	}
 	if (flags){
@@ -1918,8 +1809,7 @@ int fix_all_socket_lists()
 #ifdef USE_TCP
 	flags=0;
 	if (!tcp_disable && (fix_socket_list(&tcp_listen, &flags)!=0)){
-		LOG(L_ERR, "ERROR: fix_all_socket_lists: fix_socket_list"
-				" tcp failed\n");
+		LM_ERR("fix_socket_list tcp failed\n");
 		goto error;
 	}
 	if (flags){
@@ -1928,8 +1818,7 @@ int fix_all_socket_lists()
 #ifdef USE_TLS
 	flags=0;
 	if (!tls_disable && (fix_socket_list(&tls_listen, &flags)!=0)){
-		LOG(L_ERR, "ERROR: fix_all_socket_lists: fix_socket_list"
-				" tls failed\n");
+		LM_ERR("fix_socket_list tls failed\n");
 		goto error;
 	}
 	if (flags){
@@ -1940,8 +1829,7 @@ int fix_all_socket_lists()
 #ifdef USE_SCTP
 	flags=0;
 	if (!sctp_disable && (fix_socket_list(&sctp_listen, &flags)!=0)){
-		LOG(L_ERR, "ERROR: fix_all_socket_lists: fix_socket_list"
-				" sctp failed\n");
+		LM_ERR("fix_socket_list sctp failed\n");
 		goto error;
 	}
 	if (flags){
@@ -1959,7 +1847,7 @@ int fix_all_socket_lists()
 			&& (sctp_listen==0)
 #endif
 		){
-		LOG(L_ERR, "ERROR: fix_all_socket_lists: no listening sockets\n");
+		LM_ERR("no listening sockets\n");
 		goto error;
 	}
 	return 0;
@@ -2003,7 +1891,7 @@ void print_all_socket_lists()
 						si->flags & SI_IS_MCAST ? " mcast" : "",
 						si->flags & SI_IS_MHOMED? " mhomed" : "");
 				if (si->useinfo.name.s)
-					printf(" advertise %s", si->useinfo.name.s);
+					printf(" advertise %s:%d", si->useinfo.name.s, si->useinfo.port_no);
 				printf("\n");
 			}
 		}
@@ -2054,6 +1942,103 @@ void init_proto_order()
 			if (nxt_proto[r]==PROTO_SCTP)
 				nxt_proto[r]=nxt_proto[PROTO_SCTP];
 		}
+
+	/* Deliberately skipping PROTO_WS and PROTO_WSS here as these
+	   are just upgraded TCP and TLS connections */
 }
 
 
+/**
+ * parse '[port:]host[:port]' string to a broken down structure
+ */
+int parse_protohostport(str* ins, sr_phostp_t *r)
+{
+	char* first; /* first ':' occurrence */
+	char* second; /* second ':' occurrence */
+	char* p;
+	int bracket;
+	char* tmp;
+
+	first=second=0;
+	bracket=0;
+	memset(r, 0, sizeof(sr_phostp_t));
+
+	/* find the first 2 ':', ignoring possible ipv6 addresses
+	 * (substrings between [])
+	 */
+	for(p=ins->s; p<ins->s+ins->len; p++){
+		switch(*p){
+			case '[':
+				bracket++;
+				if (bracket>1) goto error_brackets;
+				break;
+			case ']':
+				bracket--;
+				if (bracket<0) goto error_brackets;
+				break;
+			case ':':
+				if (bracket==0){
+					if (first==0) first=p;
+					else if( second==0) second=p;
+					else goto error_colons;
+				}
+				break;
+		}
+	}
+	if (p==ins->s) return -1;
+	if (*(p-1)==':') goto error_colons;
+
+	if (first==0) { /* no ':' => only host */
+		r->host.s=ins->s;
+		r->host.len=(int)(p-ins->s);
+		goto end;
+	}
+	if (second) { /* 2 ':' found => check if valid */
+		if (parse_proto((unsigned char*)ins->s, first-ins->s, &r->proto)<0)
+			goto error_proto;
+		r->port=strtol(second+1, &tmp, 10);
+		if ((tmp==0)||(*tmp)||(tmp==second+1)) goto error_port;
+		r->host.s=first+1;
+		r->host.len=(int)(second-r->host.s);
+		goto end;
+	}
+	/* only 1 ':' found => it's either proto:host or host:port */
+	r->port=strtol(first+1, &tmp, 10);
+	if ((tmp==0)||(*tmp)||(tmp==first+1)){
+		/* invalid port => it's proto:host */
+		if (parse_proto((unsigned char*)ins->s, first-ins->s, &r->proto)<0)
+			goto error_proto;
+		r->host.s=first+1;
+		r->host.len=(int)(p-r->host.s);
+	}else{
+		/* valid port => its host:port */
+		r->host.s=ins->s;
+		r->host.len=(int)(first-r->host.s);
+	}
+end:
+	return 0;
+error_brackets:
+	LM_ERR("too many brackets in %.*s\n", ins->len, ins->s);
+	return -1;
+error_colons:
+	LM_ERR("too many colons in %.*s\n", ins->len, ins->s);
+	return -1;
+error_proto:
+	LM_ERR("bad protocol in %.*s\n", ins->len, ins->s);
+	return -1;
+error_port:
+	LM_ERR("bad port number in %.*s\n", ins->len, ins->s);
+	return -1;
+}
+
+/**
+ * lookup a local socket by '[port:]host[:port]' string
+ */
+struct socket_info* lookup_local_socket(str *phostp)
+{
+	sr_phostp_t r;
+	if(parse_protohostport(phostp, &r)<0)
+		return NULL;
+	return grep_sock_info(&r.host, (unsigned short)r.port,
+			(unsigned short)r.proto);
+}
